package ee.decomposition;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.StringTokenizer;

import ee.config.Config;
import ee.input.file.ReadFile;
import ee.utilities.StopWords;

public class Compounds {
	
	public static class CompoundsInfoComperator implements Comparator {
		public int compare(Object sf1, Object sf2) {
			CompoundsInfo sortInfo1 = (CompoundsInfo)sf1;
			CompoundsInfo sortInfo2 = (CompoundsInfo)sf2;
			
			if (sortInfo1.fp == sortInfo2.fp) {
				return 0;
			} else if (sortInfo1.fp > sortInfo2.fp) {
				return -1;
			} else {
				return 1;
			}
		}
	}
	
	public static class CompoundsInfo{
		public String compressed;
		public String phrase;
		public int fc;
		public int fp;
		public CompoundsInfo(String compressed, String phrase, int fc, int fp){
			this.compressed = compressed;
			this.phrase = phrase;
			this.fc = fc;
			this.fp = fp;
		}
	}
	
	public static void compounds() throws IOException{
		HashMap queryMap = new HashMap();
		StopWords sw = new StopWords();
		
		File f = new File(Config.PATHS_PREPROCESSES_RAW_LOG_PATH, "total_frequency");
		File outputFile = new File(Config.PATHS_PROCESSED_OUTPUT, "compounds");
		ReadFile rf = new ReadFile(f);
		
		String tempString = "";
		while((tempString = rf.getNextLine()) != null){
			tempString = tempString.replaceAll("\"", "");
			StringTokenizer st = new StringTokenizer(tempString, "\t");
			
			if(st.countTokens() ==2){
				int frequency = Integer.parseInt(st.nextToken().trim());
				String query = st.nextToken().trim();
				
				StringTokenizer st2 = new StringTokenizer(query, " ");
				while(st2.hasMoreTokens()){
					String token = st2.nextToken();
					if(!sw.isStopWord(token) && token.length() > 3){
						if(queryMap.containsKey(token)){
							queryMap.put(token, new Integer(frequency + ((Integer)queryMap.get(token)).intValue()));
						} else {
							queryMap.put(token, new Integer(frequency));
						}
					}
				}	
			}
		}
		
		tempString = "";
		rf = new ReadFile(f);
		HashMap resultMap = new HashMap();
		int numberOfQueries = 0;
		while((tempString = rf.getNextLine()) != null){
			tempString = tempString.replaceAll("\"", "");
			StringTokenizer st = new StringTokenizer(tempString, "\t");
			if(st.countTokens() == 2){
				int frequency = Integer.parseInt(st.nextToken().trim());
				String query = st.nextToken().trim();
				StringTokenizer st2 = new StringTokenizer(query, " ");
				String[] terms = new String[st2.countTokens()];
				for(int i = 0; i < terms.length; i++){
					terms[i] = st2.nextToken().trim();
				}
				
				for(int i = 0; i < terms.length-1; i++){
					StringBuffer sb = new StringBuffer();
					for(int j = i; j < terms.length; j++){
						sb.append(terms[j]);
						if(queryMap.containsKey(sb.toString()) && i != j){
							
							String compound = sb.toString();
							StringBuffer sb2 = new StringBuffer();
							for(int k = i; k <= j; k++){
								sb2.append(terms[k]);
								sb2.append(" ");
							}
							String phrase = sb2.toString().trim();
							
							if(resultMap.containsKey(compound)){
								Compounds.CompoundsInfo ci = (Compounds.CompoundsInfo)resultMap.get(compound);
								ci.fp+=frequency;
								
							} else {
								int fc = ((Integer)queryMap.get(compound)).intValue();
								int fp = ((Integer)queryMap.get(compound)).intValue();
								resultMap.put(compound, new Compounds.CompoundsInfo(compound, phrase, fc, fp));
							}
							numberOfQueries+=frequency;	
						}
					}
				}
			}
		}
		OutputStream outputStream = new BufferedOutputStream(new FileOutputStream(outputFile));
		Iterator iterator = resultMap.entrySet().iterator();
		ArrayList sortList = new ArrayList();
		
		while(iterator.hasNext()){
			Map.Entry  hm = (Map.Entry)iterator.next();
			sortList.add((CompoundsInfo)hm.getValue());
		}
		Collections.sort(sortList, new Compounds.CompoundsInfoComperator());
		
		for(int i = 0; i < sortList.size(); i++){
			CompoundsInfo cin = (CompoundsInfo)sortList.get(i);
			//if(cin.fp < cin.fc){
				outputStream.write((cin.phrase + "\t" +  cin.compressed + "\t" + cin.fp + "\t" + cin.fc + "\n").getBytes());
			//} else {
			//	outputStream.write((cin.compressed + "\t" +  cin.phrase + "\t" + cin.fc + "\t" + cin.fp + "\n").getBytes());
			//}
		}
		
		outputStream.write((numberOfQueries + " Queries \n").getBytes());
		outputStream.close();
		
		System.out.println("Compounds file created");
	}
	
	public static void main(String[] args) throws IOException{
		Compounds.compounds();
	}
}
