package ee.decomposition;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.StringTokenizer;

import ee.config.Config;
import ee.input.file.ReadFile;
import ee.utilities.StopWords;

public class Decompose {
	private HashMap phraseMap;
	private HashMap compoundMap;
	private HashMap connexityMap;
	private StopWords stopWords;
	
	private int totalQueryFrequency;
	
	public class DecomposedString{
		public int numberOfTokens;
		public String originalString;
		public String[]tokenArray; 
		
		public DecomposedString(String inputString){
			this.originalString = inputString;
		}
	}
	
	public static class MultiGramInfo{
		public int numberOfTokens;
		public String multiGram;
		public int frequency;
		
		public int score;
		//public HashMap leftNeighbour;
		//public HashMap rightNeighbour;
		//public int queryPosition;
		
		public MultiGramInfo(String multiGram, int numberOfTokens, int frequency){
			this.multiGram = multiGram;
			this.numberOfTokens = numberOfTokens;
			this.frequency = frequency;
			this.score = this.frequency;
		}
	}
	
	public static class MultiGramInfoComperator implements Comparator {
		public int compare(Object sf1, Object sf2) {
			MultiGramInfo sessionInfo1 = (MultiGramInfo)sf1;
			MultiGramInfo sessionInfo2 = (MultiGramInfo)sf2;
			
			if (sessionInfo1.score == sessionInfo2.score) {
				return 0;
			} else if (sessionInfo1.score > sessionInfo2.score) {
				return -1;
			} else {
				return 1;
			}
		}
	}
	
	public Decompose() throws IOException{
		stopWords = new StopWords();
		readPhrases();
		//readCompounds();
		connexitySegmentation();
		System.out.println("Decompose initialized");
	}
	
	private void readPhrases() throws IOException{
		File f = new File(Config.PATHS_PROCESSED_OUTPUT, "phrases");
		ReadFile rf = new ReadFile(f);
		phraseMap = new HashMap();
		
		String tempString = "";
		while((tempString = rf.getNextLine()) != null){
			StringTokenizer st = new StringTokenizer(tempString, "\t");
			if(st.countTokens() == 2){
				int freq = Integer.parseInt(st.nextToken().trim());
				String phrase = st.nextToken().trim();
				
				if(freq >= Config.DECOMPOSITION_MINIMUM_PHRASE_FREQUENCY){
					phraseMap.put(phrase, new Integer(freq));
				}
			}
		}
	}
	
	private void readCompounds() throws IOException{
		File f = new File(Config.PATHS_PROCESSED_OUTPUT, "compounds");
		if(!f.exists()){
			Compounds.compounds();
		}
		ReadFile rf = new ReadFile(f);
		compoundMap = new HashMap();
		
		String tempString = "";
		while((tempString = rf.getNextLine()) != null){
			StringTokenizer st = new StringTokenizer(tempString, "\t");
			if(st.countTokens() == 4){
				String compound = st.nextToken();
				String mergeWord = st.nextToken();
				int compoundFreq = Integer.parseInt(st.nextToken());
				int mergeWordFreq = Integer.parseInt(st.nextToken());
				
				if(compoundFreq <= Config.DECOMPOSITION_MINIMUM_COMPOUND_FREQUENCY){
					compoundMap.put(compound, new Integer(compoundFreq));
				}
			}
		}
	}
	
	private void connexitySegmentation() throws IOException{
		
		File f = new File(Config.PATHS_PREPROCESSES_RAW_LOG_PATH, "total_frequency");
		ReadFile rf = new ReadFile(f);
		connexityMap = new HashMap();
		
		String tempString = "";
		while((tempString = rf.getNextLine()) != null){
			
			StringTokenizer lst = new StringTokenizer(tempString, "\t");
			if(lst.countTokens() == 2){
				int queryFrequency = Integer.parseInt(lst.nextToken().trim());
				String query = lst.nextToken().replaceAll("\"", "").trim();
				
				StringTokenizer stq = new StringTokenizer(query, " ");
				int numberOfTokens = stq.countTokens();
				
				String tokenArray[] = new String[numberOfTokens];
				for(int i = 0; i < tokenArray.length; i++){
					tokenArray[i] = stq.nextToken();
				}
				
				for(int i = 0; i < numberOfTokens; i++){
					StringBuffer sb = new StringBuffer();
					for(int j = i; j < numberOfTokens; j++){
						//if(j > Config.DECOMPOSITION_CONNECITY_MAX_TERM_FREQUENCY){
						//	break;
						//}
						
						sb.append(tokenArray[j]);
						sb.append(" ");
						
						String compound = sb.toString().trim();
						if(!stopWords.isStopWord(compound)){
							if(connexityMap.containsKey(compound)){
								MultiGramInfo mgi = (MultiGramInfo)connexityMap.get(compound);
								mgi.frequency += queryFrequency;	
								
							} else {
								connexityMap.put(compound, new MultiGramInfo(compound, j-i+1, queryFrequency));
							}
						} 
					}
				}	
			} else {
				StringTokenizer tempStringST = new StringTokenizer(tempString, " ");
				totalQueryFrequency = Integer.parseInt(tempStringST.nextToken());
			}
		}
		// relative frequency count to remove insignificant compunds   Fi = Ri/K, Fi > K
	}
	
	public ArrayList simpleSegmentFinder(String string){
		
		StringTokenizer st = new StringTokenizer(string, " ");
		int numberOfTokens = st.countTokens();
		ArrayList resultArray = new ArrayList();
		
		String tokenArray[] = new String[numberOfTokens];
		for(int i = 0; i < numberOfTokens; i++){
			tokenArray[i] = st.nextToken();			
		}
		
		for(int i = 0; i < numberOfTokens; i++){
			StringBuffer sb = new StringBuffer();
			for(int j = i; j < numberOfTokens; j++){
				//if(j > Config.DECOMPOSITION_CONNECITY_MAX_TERM_FREQUENCY){
				//	break;
				//}
				
				sb.append(tokenArray[j]);
				String tempSegment = sb.toString().trim();
				
				
				if(j == i){
					if(connexityMap.containsKey(tempSegment)){
						MultiGramInfo mgi = (MultiGramInfo)connexityMap.get(tempSegment);
						resultArray.add(mgi);
					}
				} else {
					tempSegment = sb.toString().trim();
					if(connexityMap.containsKey(tempSegment)){
						MultiGramInfo mgi = (MultiGramInfo)connexityMap.get(tempSegment);
						
						mgi.score += mgi.frequency * mgi.numberOfTokens;
						
//						if(compoundMap.containsKey(tempSegment)){
//							int frequency = ((Integer)compoundMap.get(tempSegment)).intValue();
//							mgi.score += 2 * mgi.numberOfTokens * frequency;
//						}
						
						if(phraseMap.containsKey(tempSegment)){
							int frequency = ((Integer)phraseMap.get(tempSegment)).intValue();
							mgi.score += 3 * mgi.numberOfTokens * frequency;
						}
						
						resultArray.add(mgi);
					}

					
				}
				
				sb.append(" ");
			}
		}
		return resultArray;
	}
	
	public String findBestSegmentFromString(String string){
		ArrayList results = simpleSegmentFinder(string);
		Collections.sort(results, new MultiGramInfoComperator());
		MultiGramInfo mgi = (MultiGramInfo)results.get(0);
		return mgi.multiGram;
	}
	
	public void segmentInfoToScreen(String string){
		ArrayList results = simpleSegmentFinder(string);
		Collections.sort(results, new MultiGramInfoComperator());
		System.out.println("Segmentations results of the string: " + string);
		for(int i = 0; i < results.size(); i++){
			MultiGramInfo mgi = (MultiGramInfo)results.get(i);
			System.out.println(mgi.multiGram + "  " + mgi.score);
		}
	}
	
	public static void main(String[] args) throws IOException{
		Decompose d = new Decompose();
		d.segmentInfoToScreen("google search engine britney spears");
		d.segmentInfoToScreen("google search mega millions lottery");
		d.segmentInfoToScreen("american idol antonella barba pictures");
		d.segmentInfoToScreen("president bush");
	}
}
