package ee.evaluation;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import ee.config.Config;

public class Evaluation {	
	public static class EvaluationInfo implements Serializable{
		public String userName;
		public String query;
		public int queryNumber;
		public String overview;
		public String navigate;
		public int regularNumber;
		public int improvedNumber;
		
		public ArrayList<String> improvedSearch;
		
		public EvaluationInfo(String userName, String query, int queryNumber, String overview, String navigate, int regularNumber, int improvedNumber){
			this.userName = userName;
			this.query = query;
			this.queryNumber = queryNumber;
			this.overview = overview;
			this.navigate = navigate;
			this.regularNumber = regularNumber;
			this.improvedNumber = improvedNumber;
			improvedSearch = new ArrayList<String>();
		}
	}
	
	private HashMap<String, HashMap<Integer, EvaluationInfo>> evaluationMap;
	private File dataFile = new File(Config.PATHS_PROCESSED_OUTPUT, "evaluation_results");
	private File dataFile2 = new File(Config.PATHS_PROCESSED_OUTPUT, "evaluation_result_fields");

	public Evaluation() throws IOException, ClassNotFoundException{
		if(!dataFile.exists()){
			evaluationMap = new HashMap<String, HashMap<Integer, EvaluationInfo>>();
		} else {
			readResultsFromFile();
		}
	}
	
	public synchronized void addResults(EvaluationInfo ei) throws IOException{
		if(evaluationMap.containsKey(ei.userName)){
			HashMap<Integer, EvaluationInfo> userMap = evaluationMap.get(ei.userName);
			userMap.put(ei.queryNumber, ei);
		} else {
			HashMap<Integer, EvaluationInfo> userMap = new HashMap<Integer, EvaluationInfo>();
			userMap.put(ei.queryNumber, ei);
			evaluationMap.put(ei.userName, userMap);
		}
		writeResultsToFile();
	}
	
	public synchronized HashMap<Integer, EvaluationInfo> getResults(String userName){
		return evaluationMap.get(userName);
	}
	
	public synchronized void writeResultsToFile() throws FileNotFoundException, IOException{
		ObjectOutputStream oos = new ObjectOutputStream(new FileOutputStream(dataFile));
        oos.writeObject(evaluationMap);
        oos.close();
        writeResultsToFile2();
	}
	
	public synchronized void writeResultsToFile2() throws IOException{
		OutputStream os = new BufferedOutputStream(new FileOutputStream(dataFile2));

		for(Map.Entry<String, HashMap<Integer, EvaluationInfo>> resultMap : evaluationMap.entrySet()){
			if(resultMap.getValue().size() != 10){
				continue;
			}
			
			os.write((resultMap.getKey() + "\n").getBytes());
			
			for(Map.Entry<Integer, EvaluationInfo> query : resultMap.getValue().entrySet()){
				EvaluationInfo ie = query.getValue();
				os.write(("query number "+ie.queryNumber+"\n").getBytes());
				os.write(("navigate help : "+ie.navigate+"\n").getBytes());
				os.write(("navigation overview : "+ie.overview+"\n").getBytes());
				
				os.write(("improved search\n").getBytes());
				for(int i = 0; i < ie.improvedSearch.size(); i++){
					String field = ie.improvedSearch.get(i);
					if("not".equals(field)){
						field = "not relevant";
					} else if ("good".equals(field)){
						field = "match";
					} else if ("relevant".equals(field)){
						field = "new article";
					}
					os.write((" " + (i + 1) + " : " + field).getBytes());
					if((i+1) % 5 == 0 && (i+1) != ie.improvedSearch.size()){
					os.write(("\n").getBytes());
					}
				}
				os.write(("\n").getBytes());
				
				os.write(("unique queries in regular serach: " + ie.regularNumber + "\n").getBytes());
				os.write(("unique queries in improved serach: " + ie.improvedNumber + "\n\n").getBytes());
			}
		}
		
		os.close();
	}
	
	public void readResultsFromFile() throws IOException, ClassNotFoundException{
        ObjectInputStream ois = new ObjectInputStream(new FileInputStream(dataFile));
        evaluationMap = (HashMap)ois.readObject();
        ois.close();
	}
	
	public HashMap<String, HashMap<Integer, EvaluationInfo>> getEvaluationMap(){
		return evaluationMap;
	}
}
