package ee.input.preprocessing;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import ee.config.Config;
import ee.input.file.ReadFile;
import ee.utilities.StopWords;

public class FilePreprocessing {
	
	public static class SortInfoComperator implements Comparator {
		public int compare(Object sf1, Object sf2) {
			SortInfo sortInfo1 = (SortInfo)sf1;
			SortInfo sortInfo2 = (SortInfo)sf2;
			
			if (sortInfo1.frequency == sortInfo2.frequency) {
				return 0;
			} else if (sortInfo1.frequency > sortInfo2.frequency) {
				return -1;
			} else {
				return 1;
			}
		}
	}
	
	public static class SortInfo{
		public int frequency;
		public String query;
		
		public SortInfo(String query, int frequency){
			this.frequency = frequency;
			this.query = query;
		}
	}
	
	public static void analyze(File file, StopWords sw) throws IOException{
		ReadFile rf = new ReadFile(file);
		File outputFile1 = new File(Config.PATHS_PREPROCESSES_RAW_LOG_PATH, "processed_" + file.getName().replaceAll(":", "-"));
		File outputFile2 = new File(Config.PATHS_PREPROCESSES_RAW_LOG_PATH, "frequency_" + file.getName().replaceAll(":", "-"));
		outputFile1.getParentFile().mkdirs();
		OutputStream outputStream1 = new BufferedOutputStream(new FileOutputStream(outputFile1));
		
		String pattern = "([\\da-f]{32})(.*)[\\&\\?][pva]{1,2}\\=([^\\&\\^]*)[\\&^].*";
		
		Pattern p = Pattern.compile(pattern);
		Pattern p2 = Pattern.compile("\\%22");
		Pattern p3 = Pattern.compile("\\%2E");
		Pattern p8 = Pattern.compile("\\%27");
		Pattern p4 = Pattern.compile("\\%[A-F\\d]{2}");
		Pattern p7 = Pattern.compile("'s");
		Pattern p5 = Pattern.compile("[^\\w\\.\"]");
		Pattern p6 = Pattern.compile("  ");
		
		HashMap queryMap = new HashMap();
		
		String tempString = ""; 
		int numberOfQueries = 0;
		while((tempString = rf.getNextLine()) != null){
			
			Matcher m = p.matcher(tempString);
			if(m.find()){
				String grp1str = m.group(1);
				String userID = grp1str.substring(0, 8);
				int time = Integer.parseInt(grp1str.substring(8, 16), 16);
				String query = m.group(3);				
				String typeTemp = m.group(2);
				
				if(query.trim().equals("")){
					continue;
				}
				
				if(query.length() > Config.PREPROCESSING_MAX_CHARACTERS){
					continue;
				}
				
				String type = "";
				if(!Config.PREPROCESSING_RAW_LOG_ALL){
					if(typeTemp.indexOf("/rss") >= 0){
						if(!Config.PREPROCESSING_RAW_LOG_RSS) continue;
						type = "rss";
					} else if(typeTemp.indexOf("/usns") >= 0){
						if(!Config.PREPROCESSING_RAW_LOG_USNS) continue;
						type = "usns";	
					} else if(typeTemp.indexOf("/search") >= 0){
						if(!Config.PREPROCESSING_RAW_LOG_SEARCH) continue;
						type = "search";
					} 
				}
				
				Matcher m2 = p2.matcher(query);
				query = m2.replaceAll("\"");
				Matcher m3 = p3.matcher(query);
				query = m3.replaceAll(".");
				Matcher m8 = p8.matcher(query);
				query = m8.replaceAll("'");
				Matcher m4 = p4.matcher(query);
				query = m4.replaceAll(" ");
				Matcher m7 = p7.matcher(query);
				query = m7.replaceAll("");
				Matcher m5 = p5.matcher(query);
				query = m5.replaceAll(" ");
				Matcher m6 = p6.matcher(query);
				query = m6.replaceAll(" ");
				query = query.toLowerCase();
				query = query.trim();
				
				if(sw.isStopWord(query)){
					continue;
				}
				numberOfQueries++;
				
				if(queryMap.containsKey(query)){
					Integer newCount = (Integer)queryMap.get(query);
					queryMap.put(query, new Integer(newCount.intValue() + 1));
				} else {
					queryMap.put(query, new Integer(1));
				}
							
//				System.out.println(tempString);
//				System.out.println(userID + "\t: " + time + "  \t: " + type + "\t: " + query);
				outputStream1.write((userID + "\t" + time + "\t" + type + "  \t" + query + "\n").getBytes());
								
			} 

		}
		
		outputStream1.close();
		
		FilePreprocessing.simpleQueryFrequencyToFile(queryMap, outputFile2, numberOfQueries);
		
		System.out.println(file + "  " + outputFile2.getName() + "  created");
	}
	
	public static void simpleQueryFrequencyToFile(HashMap sortInfoMap, File file, int numberOfQueries) throws IOException{
		OutputStream outputStream = new BufferedOutputStream(new FileOutputStream(file));
		
		Iterator iterator = sortInfoMap.entrySet().iterator();
		ArrayList sortList = new ArrayList();
		
		while(iterator.hasNext()){
			Map.Entry  hm = (Map.Entry)iterator.next();
			sortList.add(new SortInfo((String)hm.getKey(), ((Integer)hm.getValue()).intValue()));
		}
		
		Collections.sort(sortList, new SortInfoComperator());
		
		for(int i = 0; i < sortList.size(); i++){
			SortInfo si = (SortInfo)sortList.get(i);
			outputStream.write((si.frequency + "\t" + si.query + "\n").getBytes());
		}
		
		outputStream.write((numberOfQueries + " Queries \n").getBytes());
		
		outputStream.close();
		
	}
	
}
