package ee.session;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Map;
import java.util.StringTokenizer;

import ee.config.Config;
import ee.decomposition.Phrases;
import ee.input.file.FileFinder;
import ee.input.file.ReadFile;

public class SessionExtraction {
	
	public static class SessionInfo {	
		public String id;
		public int time;
		public String query;
		public SessionInfo(String id, int time, String query){
			this.id = id;
			this.time = time;
			this.query = query;
		}
	}
	
	public static class SessionInfoComperator implements Comparator {
		public int compare(Object sf1, Object sf2) {
			SessionInfo sessionInfo1 = (SessionInfo)sf1;
			SessionInfo sessionInfo2 = (SessionInfo)sf2;
			
			if (sessionInfo1.time == sessionInfo2.time) {
				return 0;
			} else if (sessionInfo1.time < sessionInfo2.time) {
				return -1;
			} else {
				return 1;
			}
		}
	}
	
	public static void findSessions() throws IOException{
		ArrayList files = FileFinder.getAllFiles(Config.PATHS_PREPROCESSES_RAW_LOG_PATH, "processed");
		File tempFile = new File(Config.PATHS_PREPROCESSES_RAW_LOG_PATH, "session.tmp");
		tempFile.getParentFile().mkdirs();
		OutputStream tempFileStream = new BufferedOutputStream(new FileOutputStream(tempFile));
		
		// EXTRACT SESSIONS
		
		HashMap sessionMap = new HashMap();
		int numberOfSessions = 0;
		
		for(int i = Config.CORRELATION_SKIP_FROM_START; i < Config.CORRELATION_TIME_DURATION + Config.CORRELATION_SKIP_FROM_START; i++){
			File f = (File)files.get(i);
			ReadFile rf = new ReadFile(f);
			
			String tempString = "";
			while((tempString = rf.getNextLine()) != null){
				
				StringTokenizer st = new StringTokenizer(tempString, "\t");
				if(st.countTokens() ==4){
					String id = st.nextToken();
					int time = Integer.parseInt(st.nextToken());
					st.nextToken();
					String query = st.nextToken();
					SessionInfo si = new SessionInfo(id, time, query);
					
					if(sessionMap.containsKey(id)){
						LinkedList tempList = (LinkedList)sessionMap.get(si.id);
						SessionInfo tempInfo = (SessionInfo)tempList.getLast();
						if((si.time - tempInfo.time) <= Config.SESSIONS_SESSION_TIME_FRAME){
							tempList.add(si);
						} else {
							if(tempList.size() <= 1){
								if(tempList.size() == 1){
									tempList.remove(0);
								}
								tempList.add(si);
							} else {
								int size = tempList.size();
								for(int j = 0; j < size; j++){
									SessionInfo siOutput = (SessionInfo)tempList.removeFirst();
									tempFileStream.write((siOutput.id + "\t" + siOutput.time + "\t" + siOutput.query + "\n").getBytes());
								}
								numberOfSessions++;
								tempList.add(si);
							}
						}
					} else {
						LinkedList singleSessionList = new LinkedList();
						singleSessionList.add(si);
						sessionMap.put(si.id, singleSessionList);
					}
				}
			}
		}
		
		Iterator iterator = sessionMap.entrySet().iterator();
		
		while(iterator.hasNext()){
			Map.Entry  hm = (Map.Entry)iterator.next();
			LinkedList ll = (LinkedList)hm.getValue();
			int sizell = ll.size();
			if(sizell > 1){
				numberOfSessions++;
				for(int i = 0; i < sizell; i++){
					SessionInfo si = (SessionInfo)ll.removeFirst();
					tempFileStream.write((si.id + "\t" + si.time + "\t" + si.query + "\n").getBytes());
				}
			}
		}
		
		tempFileStream.write((numberOfSessions + "  Sessions").getBytes());
		tempFileStream.close();
		
		// CLEANUP - REMOVING DUPLICATES AND TOO FEW/TOO MANY
		
		File outputFile = new File(Config.PATHS_PREPROCESSES_RAW_LOG_PATH, "session");
		OutputStream outputStream = new BufferedOutputStream(new FileOutputStream(outputFile));
		ReadFile rf = new ReadFile(tempFile);
		Phrases phrases = new Phrases();
		
		numberOfSessions = 0;
		HashMap tempMap = new HashMap();
		String tempString = "";
		String sessionID = null;
		while((tempString = rf.getNextLine()) != null){
			
			StringTokenizer st = new StringTokenizer(tempString, "\t");
			if(st.countTokens() == 3){
				String id = st.nextToken();
				int time = Integer.parseInt(st.nextToken());
				String query = st.nextToken();
				SessionInfo si = new SessionInfo(id, time, query);
				
				if(sessionID == null){
					sessionID = id;
				}
				
				if(sessionID.equals(si.id)){
					if(!tempMap.containsKey(si.query)){
						tempMap.put(si.query, new Integer(si.time));
					}
				} else {
					if(tempMap.size() > 1 && tempMap.size() < Config.SESSIONS_NUMBER_OF_QUERY_REFINEMENTS){
						numberOfSessions++;
						ArrayList sortArray = new ArrayList();
						Iterator it = tempMap.entrySet().iterator();
						
						while(it.hasNext()){
							Map.Entry element = (Map.Entry)it.next();
							String q = (String)element.getKey();
							int t = ((Integer)element.getValue()).intValue();
							StringTokenizer stq = new StringTokenizer(q, " ");
							if(stq.countTokens() <= Config.PREPROCESSING_MAX_TERMS){
								sortArray.add(new SessionInfo(sessionID, t, q));
								phrases.addPhrase(q);
							}
						}
						
						Collections.sort(sortArray, new SessionInfoComperator());
			
						for(int i = 0; i < sortArray.size(); i++){
							SessionInfo sessionInfo = (SessionInfo)sortArray.get(i);
							outputStream.write((sessionInfo.id + "\t" + sessionInfo.time + "\t" + sessionInfo.query + "\n").getBytes());
						}
					}
					
					tempMap = new HashMap();
					tempMap.put(si.query, new Integer(si.time));
					sessionID = si.id;
				}
			}	
		}
		System.out.println(numberOfSessions + " sessions extracted");
		outputStream.write((numberOfSessions + " Sessions\n").getBytes());
		outputStream.close();
		tempFile.delete();
		phrases.writePhraseMap();
		
	}
	
	public static void main(String[] args) throws IOException{
		SessionExtraction.findSessions();
	}
}
