package ee.session;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.StringTokenizer;

import ee.config.Config;
import ee.decomposition.Decompose;
import ee.decomposition.Decompose.MultiGramInfo;
import ee.input.file.ReadFile;
import ee.input.preprocessing.FilePreprocessing.SortInfo;
import ee.input.preprocessing.FilePreprocessing.SortInfoComperator;
import ee.session.SessionExtraction.SessionInfo;
import ee.temporalcorrelation.TemporalCorrelationPreprocessing;
import ee.utilities.StopWords;

public class SessionFiltering {
	private StopWords sw;
	private HashMap groupMap = new HashMap();
	private Decompose decompose;
	
	public SessionFiltering(Decompose decompose) throws IOException{
		this.decompose = decompose;
		sw = new StopWords();	
		filterSessions();
	}
	
	public void filterSessions() throws IOException{
		File inputFile = new File(Config.PATHS_PREPROCESSES_RAW_LOG_PATH, "session");
		ReadFile rf = new ReadFile(inputFile);
		File outputFile = new File(Config.PATHS_PROCESSED_OUTPUT, "filtered_sessions");
		OutputStream outputStream = new BufferedOutputStream(new FileOutputStream(outputFile));
		
		String tempString = "";
		String sessionID = null;
		ArrayList tempArray = new ArrayList();
		int numberofSessions = 0;
		
		while((tempString = rf.getNextLine()) != null){
			tempString = tempString.replaceAll("\"", "");
			
			StringTokenizer st = new StringTokenizer(tempString, "\t");
			if(st.countTokens() == 3){
				String id = st.nextToken();
				int time = Integer.parseInt(st.nextToken());
				String query = st.nextToken().trim();
				
				if(sessionID == null){
					sessionID = id;
				}
				
				if(!sessionID.equals(id)){
					ArrayList output = overlap(tempArray);
					if(output != null){
						
						// do more stuff to each session here
						
						// find the main term that is being expanded, e.g. the msot frequenct or distinct term in the grp
						// calculate if words move in position
						// bruke levenstein for  finne feilstavelser av samme ord
						
						// finne den mest frekvente termen i hver sessiongruppe og merge p den termen
						
						numberofSessions++;
						
						for(int j = 0; j < output.size(); j++){
							SessionInfo si = (SessionInfo)output.get(j);
							outputStream.write((si.id + "\t" + si.time + "\t" + si.query + "\n").getBytes());
						}
					}
					tempArray = new ArrayList();
					tempArray.add(new SessionInfo(id, time, query));
					sessionID = id;
				} else {
					tempArray.add(new SessionInfo(id, time, query));
				}
			}	
		}
		System.out.println(numberofSessions + " filtered sessions");
		
		outputStream.write((numberofSessions + " Sessions\n").getBytes());
		outputStream.close();
		
		//writeSessionGroupsToFile("before segment merge");
		mergeSessionGroupsBySegmentationBasedMergeTerm();
		writeSessionGroupsToFile(" segment merge");
	
	}
	
	private ArrayList overlap(ArrayList inputArray){
		HashMap wordMap = new HashMap();
		
		for(int i = 0; i < inputArray.size(); i++){
			SessionInfo si = (SessionInfo)inputArray.get(i);
			StringTokenizer st = new StringTokenizer(si.query, " ");
			while(st.hasMoreTokens()){
				String s = st.nextToken();
				if(wordMap.containsKey(s)){
					wordMap.put(s, new Integer(((Integer)wordMap.get(s)).intValue()+1));
				} else {
					wordMap.put(s, new Integer(1));
				}
			}
		}
		
		Iterator iterator = wordMap.entrySet().iterator();
		String term = "";
		int maxFrequency = 0;
		while(iterator.hasNext()){
			Map.Entry me = (Map.Entry)iterator.next();
			int tempNumber = ((Integer)me.getValue()).intValue();
			String tempString = (String)me.getKey();
			if(tempNumber > maxFrequency && !sw.isStopWord(tempString)){
				term = tempString;
				maxFrequency = tempNumber;
			}
		}
		
		ArrayList outputArray = new ArrayList();
		for(int i = 0; i < inputArray.size(); i++){
			SessionInfo si = (SessionInfo)inputArray.get(i);
			if(si.query.indexOf(term) >= 0){
				outputArray.add(si);
			}
		}
		
		if(outputArray.size() > 1){
			Collections.sort(outputArray, new SessionExtraction.SessionInfoComperator());
			
			// GROUP ALL TERMS
			insertSessionGroupByTerm(outputArray, term);
			return outputArray;
		}
		
		return null;
	}
	
	private void insertSessionGroupByTerm(ArrayList groupArray, String mergeTerm){
		
		if(groupMap.containsKey(mergeTerm)){
			HashMap groupItemMap = (HashMap)groupMap.get(mergeTerm);
			
			for(int i = 0; i < groupArray.size(); i++){
				SessionInfo si = (SessionInfo)groupArray.get(i);
				if(groupItemMap.containsKey(si.query)){
					groupItemMap.put(si.query, new Integer(((Integer)groupItemMap.get(si.query)).intValue() + 1));
				} else {
					groupItemMap.put(si.query, new Integer(1));
				}
			}
		} else {
			HashMap groupItemMap = new HashMap();
			for(int i = 0; i < groupArray.size(); i++){
				SessionInfo si = (SessionInfo)groupArray.get(i);
				groupItemMap.put(si.query, new Integer(1));
			}
			groupMap.put(mergeTerm, groupItemMap);
		}	
		
	}
	
	private void mergeSessionGroupsBySegmentationBasedMergeTerm(){
		
		HashMap newGroupMap = new HashMap();
		
		Iterator itg = groupMap.entrySet().iterator();
		while(itg.hasNext()){
			Map.Entry meg = (Map.Entry)itg.next();
			String mergeTerm = (String)meg.getKey();
			HashMap groupQueryMap = (HashMap)meg.getValue();
			
			HashMap segmentMap = new HashMap();
			Iterator itq = groupQueryMap.entrySet().iterator();
			while(itq.hasNext()){
				Map.Entry meq = (Map.Entry)itq.next();
				String query = (String)meq.getKey();
				
				ArrayList segmentArray = decompose.simpleSegmentFinder(query);
				for(int i = 0; i < segmentArray.size(); i++){
					Decompose.MultiGramInfo mgi = (MultiGramInfo)segmentArray.get(i);
					if(segmentMap.containsKey(mgi.multiGram)){
						segmentMap.put(mgi.multiGram, new Integer(((Integer)segmentMap.get(mgi.multiGram)).intValue()+mgi.score));
					} else {
						segmentMap.put(mgi.multiGram, new Integer(mgi.score));
					}
				}
			}
			
			ArrayList tempSortArray = new ArrayList();
			Iterator its = segmentMap.entrySet().iterator();
			while(its.hasNext()){
				Map.Entry mes = (Map.Entry)its.next();
				tempSortArray.add(new SortInfo((String)mes.getKey(), ((Integer)mes.getValue()).intValue()));
			}
			Collections.sort(tempSortArray, new SortInfoComperator());
			
			String mergeSegment = ((SortInfo)tempSortArray.get(0)).query;
			
			if(!(mergeSegment.indexOf(mergeTerm) >= 0)){
				mergeSegment = mergeTerm;
			}
			
			if(newGroupMap.containsKey(mergeSegment)){
				HashMap existingMap = (HashMap)newGroupMap.get(mergeSegment);
				
				itq = groupQueryMap.entrySet().iterator();
				while(itq.hasNext()){
					Map.Entry meq = (Map.Entry)itq.next();
					String query = (String)meq.getKey();
					Integer freq = (Integer)meq.getValue();
					
					if(existingMap.containsKey(query)){
						existingMap.put(query, new Integer(((Integer)existingMap.get(query)).intValue() + freq.intValue()));
					} else {
						existingMap.put(query, freq);
					}
				}
				
			} else {
				newGroupMap.put(mergeSegment, groupQueryMap);
			}
		}
		
		groupMap = newGroupMap;
	}
	
	public HashMap getSessionQueryGroupMap(){
		return groupMap;
	}
	
	public HashMap getSessionTermGroupMap(){
		HashMap groupTermMap = new HashMap();
		Iterator it = groupMap.entrySet().iterator();
		while(it.hasNext()){
			HashMap newTermGroup = new HashMap();
			Map.Entry me = (Map.Entry)it.next();
			String keyTerm = (String)me.getKey();
			HashMap termGroup = (HashMap)me.getValue();
			
			Iterator it2 = termGroup.entrySet().iterator();
			while(it2.hasNext()){
				Map.Entry me2 = (Map.Entry)it2.next();
				String q = (String)me2.getKey();
				int freq = ((Integer)me2.getValue()).intValue();
				newTermGroup.put(q, new Integer(freq));
				
				StringTokenizer st = new StringTokenizer(q, " ");
				while(st.hasMoreElements()){
					String s = st.nextToken();
					if(sw.isStopWord(s)){
						continue;
					}
					
					if(newTermGroup.containsKey(s)){
						newTermGroup.put(s, new Integer(((Integer)newTermGroup.get(s)).intValue() + freq));
					} else {
						newTermGroup.put(s, new Integer(freq));
					}
				}
			}
			groupTermMap.put(keyTerm, newTermGroup);
		}
		return groupTermMap;
	}
	
	public void writeSessionGroupsToFile(String partOfFileName) throws IOException{
		File outputFile = new File(Config.PATHS_PROCESSED_OUTPUT, "session_term_groups" + partOfFileName + Config.createFileInfoString());
		OutputStream outputStream = new BufferedOutputStream(new FileOutputStream(outputFile));
		
		Iterator it = groupMap.entrySet().iterator();
		while(it.hasNext()){
			Map.Entry me = (Map.Entry)it.next();
			String groupTerm = (String)me.getKey();
			HashMap queryMap = (HashMap)me.getValue();
			
			outputStream.write(("\n" + groupTerm + "\n").getBytes());
			
			Iterator it2 = queryMap.entrySet().iterator();
			while(it2.hasNext()){
				Map.Entry me2 = (Map.Entry)it2.next();
				outputStream.write(((String)me2.getKey() + "\t" + ((Integer)me2.getValue()).intValue() + "\n").getBytes());
			}
		}
		outputStream.close();
		System.out.println("written file: " + outputFile.getName());
	}
	
	public static void main(String args[]) throws IOException{
		Decompose d = new Decompose();
		Config.setCorrelationParameters(24, 1, 0, 5, 10, 2);
		SessionFiltering sf = new SessionFiltering(d);
	}
}
