package ee.temporalcorrelation;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.StringTokenizer;

import ee.decomposition.Decompose;
import ee.decomposition.Decompose.MultiGramInfo;
import ee.decomposition.Decompose.MultiGramInfoComperator;
import ee.session.SessionFiltering;
import ee.temporalcorrelation.TemporalCorrelationPreprocessing.CorrelationFields;
import ee.temporalcorrelation.TemporalCorrelationPreprocessing.CorrelationInfo;
import ee.temporalcorrelation.TemporalCorrelationPreprocessing.CorrelationInfoComperator;
import ee.temporalcorrelation.TemporalCorrelationPreprocessing.CorrelationValueComperator;
import ee.utilities.StopWords;

public class FilterCorrelations {
	
	public static class CorrelationMapInfo{
		String query;
		String mergeSegment;
		int totalGroupQueries;
		HashMap correlationMap;
		public CorrelationMapInfo(String query, String mergeSegment, int totalGroupQueries){
			this.query = query;
			this.mergeSegment = mergeSegment;
			correlationMap = new HashMap();
			this.totalGroupQueries = totalGroupQueries;
		}
	}
	
	private ArrayList correlationResultArray;
	private SessionFiltering sf;
	private Decompose decompose;
	
	public FilterCorrelations(ArrayList correlationResultArray, Decompose decompose, SessionFiltering sessionFiltering) throws IOException{
		this.correlationResultArray = correlationResultArray;
		this.decompose = decompose;
		this.sf = sessionFiltering;
	}
	
	public ArrayList filterCorrelationsWithSessionQueries() throws IOException{
		StopWords sw = new StopWords();
		HashMap termGroupMap = sf.getSessionQueryGroupMap();
		ArrayList resultArray = new ArrayList();
		
		for(int i = 0; i < correlationResultArray.size(); i++){
			CorrelationInfo ci = (CorrelationInfo)correlationResultArray.get(i);
			StringTokenizer st = new StringTokenizer(ci.query, " ");
			while(st.hasMoreTokens()){
				String tempToken = st.nextToken().trim();
				if(sw.isStopWord(tempToken)){
					continue;
				}
				
				if(termGroupMap.containsKey(tempToken)){
					ArrayList queryList = new ArrayList();
					HashMap groupMap = (HashMap)termGroupMap.get(tempToken);
					if(groupMap.containsKey(ci.query)){
						for(int j = 0; j < ci.correlationList.size(); j++){
							CorrelationFields cp  = (CorrelationFields)ci.correlationList.get(j);
							if(groupMap.containsKey(cp.query)){
								queryList.add(cp);
							}
						}
						if(queryList.size() > 1){
							CorrelationInfo rci = new CorrelationInfo(ci.query);
							rci.correlationList = queryList;
							rci.mergeSegment = tempToken;
							resultArray.add(rci);
						}
						break;
					}
				}
			}
		}
		return resultArray;
	}
	
	public ArrayList filterCorrelationsWithSessionQueriesAndSegmenation() throws IOException{
		StopWords sw = new StopWords();
		HashMap termGroupMap = sf.getSessionQueryGroupMap();
		ArrayList resultArray = new ArrayList();
		
		for(int i = 0; i < correlationResultArray.size(); i++){
			CorrelationInfo ci = (CorrelationInfo)correlationResultArray.get(i);
			
			ArrayList segmentArray = decompose.simpleSegmentFinder(ci.query);
			Collections.sort(segmentArray, new MultiGramInfoComperator());
			for(int j = 0; j < segmentArray.size(); j++){
				MultiGramInfo mgi = (MultiGramInfo)segmentArray.get(j);
				
				if(sw.isStopWord(mgi.multiGram)){
					continue;
				}
				
				if(termGroupMap.containsKey(mgi.multiGram)){ //? gives more merging but maybe too agressive
					ArrayList queryList = new ArrayList();
					HashMap groupMap = (HashMap)termGroupMap.get(mgi.multiGram);
					int totalQueryFrequency = 0;
					
					if(groupMap.containsKey(ci.query)){
						for(int k = 0; k < ci.correlationList.size(); k++){
							CorrelationFields cp  = (CorrelationFields)ci.correlationList.get(k);
							if(groupMap.containsKey(cp.query)){
								cp.frequency = ((Integer)groupMap.get(cp.query)).intValue();
								totalQueryFrequency += cp.frequency;
								queryList.add(cp);
							}
						}
						if(queryList.size() > 0){
							CorrelationInfo rci = new CorrelationInfo(ci.query);
							rci.correlationList = queryList;
							rci.mergeSegment = mgi.multiGram;
							rci.totalQueryFrequency = totalQueryFrequency;
							resultArray.add(rci);
						}
						break;
					} //?
				}
			}
		}
		return resultArray;
	}
	
	public ArrayList filterCorrelationsWithSessionTerms() throws IOException{
		StopWords sw = new StopWords();
		HashMap termGroupMap = sf.getSessionTermGroupMap();
		ArrayList resultArray = new ArrayList();
		
		for(int i = 0; i < correlationResultArray.size(); i++){
			CorrelationInfo ci = (CorrelationInfo)correlationResultArray.get(i);
			StringTokenizer st = new StringTokenizer(ci.query, " ");
			while(st.hasMoreTokens()){
				String tempToken = st.nextToken().trim();
				if(sw.isStopWord(tempToken)){
					continue;
				}
				
				if(termGroupMap.containsKey(tempToken)){
					ArrayList queryList = new ArrayList();
					HashMap groupMap = (HashMap)termGroupMap.get(tempToken);
					if(groupMap.containsKey(ci.query)){
						for(int j = 0; j < ci.correlationList.size(); j++){
							CorrelationFields cp  = (CorrelationFields)ci.correlationList.get(j);
							StringTokenizer cpst = new StringTokenizer(cp.query, " ");
							while(cpst.hasMoreTokens()){
								String tempFragment = cpst.nextToken();
								if(sw.isStopWord(tempFragment)){
									continue;
								}
								if(groupMap.containsKey(tempFragment)){
									queryList.add(cp);
									break;
								}
							}

						}
						if(queryList.size() > 0){
							CorrelationInfo rci = new CorrelationInfo(ci.query);
							rci.correlationList = queryList;
							rci.mergeSegment = tempToken;
							resultArray.add(rci);
						}
						break;
					}
				}
			}
		}
		return resultArray;
	}
	
	public static ArrayList mergeCorrelationGroups(ArrayList correlationArray){
		
		HashMap correlationMap = new HashMap();
		
		for(int i = 0; i < correlationArray.size(); i++){
			CorrelationInfo ci = (CorrelationInfo)correlationArray.get(i);
			
			CorrelationMapInfo cmi;
			if(correlationMap.containsKey(ci.mergeSegment)){
				cmi = (CorrelationMapInfo)correlationMap.get(ci.mergeSegment);
				//cmi.totalGroupQueries += ci.totalQueryFrequency;
			} else {
				cmi = new CorrelationMapInfo(ci.query, ci.mergeSegment, ci.totalQueryFrequency);
			}
			
			for(int j = 0; j < ci.correlationList.size(); j++){
				CorrelationFields cf = (CorrelationFields)ci.correlationList.get(j);
				cmi.correlationMap.put(cf.query, cf);
			}
			
			if(!correlationMap.containsKey(ci.mergeSegment)){
				correlationMap.put(ci.mergeSegment, cmi);
			}
		}
		
		ArrayList resultArray = new ArrayList();
		Iterator it = correlationMap.entrySet().iterator();
		
		while(it.hasNext()){
			Map.Entry me = (Map.Entry)it.next();
			CorrelationMapInfo cmi = (CorrelationMapInfo)me.getValue();
			
			CorrelationInfo ci = new CorrelationInfo("");
			ci.mergeSegment = cmi.mergeSegment;
			//ci.totalQueryFrequency = cmi.totalGroupQueries;
			
			Iterator itq = cmi.correlationMap.entrySet().iterator();
			while(itq.hasNext()){
				Map.Entry meq = (Map.Entry)itq.next();
				CorrelationFields cf = (CorrelationFields)meq.getValue();
				ci.correlationList.add(cf);
				ci.totalQueryFrequency += cf.frequency;
			}

			Collections.sort(ci.correlationList, new CorrelationValueComperator());
			if(ci.correlationList.size() > 1){
				resultArray.add(ci);
			}
		}
		
		Collections.sort(resultArray, new CorrelationInfoComperator());

		return resultArray;
	}
	
	public ArrayList trimCorrelationGroups(ArrayList inputList){
		ArrayList outputList = new ArrayList();
	
		for(int i = 0; i < inputList.size(); i++){
			CorrelationInfo ci = (CorrelationInfo)inputList.get(i);
			ArrayList newCorrelationFieldsList = new ArrayList();
			
			ArrayList segmentList = decompose.simpleSegmentFinder(ci.mergeSegment);
			for(int j = 0; j < ci.correlationList.size(); j++){
				CorrelationFields cf = (CorrelationFields)ci.correlationList.get(j);
				
				boolean containsMergeSegment = false;
				for(int k = 0; k < segmentList.size(); k++){
					
					if(cf.query.indexOf(((MultiGramInfo)segmentList.get(k)).multiGram) >= 0){
						containsMergeSegment = true;
					}
					
				}
				
				if(containsMergeSegment){
					newCorrelationFieldsList.add(cf);
				}
			}
			
			ci.correlationList = newCorrelationFieldsList;
			if(ci.correlationList.size() > 1){
				outputList.add(ci);
			}
		}
		
		
		return outputList;
	}

}
