package ee.temporalcorrelation;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import ee.config.Config;
import ee.temporalcorrelation.TemporalCorrelationPreprocessing.CorrelationFields;
import ee.temporalcorrelation.TemporalCorrelationPreprocessing.CorrelationInfo;
import ee.temporalcorrelation.TemporalCorrelationPreprocessing.CorrelationTotalFrequencyComperator;
import ee.temporalcorrelation.TemporalCorrelationPreprocessing.CorrelationValueComperator;
import ee.temporalcorrelation.TemporalCorrelationPreprocessing.TimeUnit;

public class WriteCorrelationData {
	
	public static float normalizationValue = 0f;
	
	public static void writeCorrelationsReportOutput(ArrayList cra, String name) throws IOException{
		File file = new File(Config.PATHS_PROCESSED_OUTPUT, name);
		OutputStream outputStream = new BufferedOutputStream(new FileOutputStream(file));
		int correlationGroups = 0;
		
		for(int i = 0; i < cra.size(); i++){
			CorrelationInfo ci = (CorrelationInfo)cra.get(i);
			
			if(ci.correlationList.size() > 0){
				correlationGroups++;
				
				if(!"".equals(ci.query.trim())){
					outputStream.write((ci.query + "\n").getBytes());
				}
				outputStream.write(("group name: " + "\t" + ci.mergeSegment + "\n").getBytes());
				
				ArrayList nyCorrelationList = new ArrayList();
				
				for(int j = 0; j < ci.correlationList.size(); j++){
					CorrelationFields cf = (CorrelationFields)ci.correlationList.get(j);
					CorrelationFields ny = new CorrelationFields(cf.query, cf.correlationValue*cf.frequency);
					nyCorrelationList.add(ny);
				}
				
				Collections.sort(nyCorrelationList, new CorrelationValueComperator());
				
				if(WriteCorrelationData.normalizationValue == 0){
					CorrelationFields cfTemp = (CorrelationFields)nyCorrelationList.get(0);
					WriteCorrelationData.normalizationValue = cfTemp.correlationValue;
					System.out.println(WriteCorrelationData.normalizationValue);
				}
				
				for(int j = 0; j < nyCorrelationList.size(); j++){		
					CorrelationFields cf = (CorrelationFields)nyCorrelationList.get(j);
					outputStream.write((1000 * (cf.correlationValue / WriteCorrelationData.normalizationValue) + "\t" + cf.query + "\n").getBytes());
				}
			}
			
			outputStream.write(("\n").getBytes());
			//outputStream.write(("Number of correlating queries found" + "\t" + ci.correlationList.size() + "\n\n").getBytes());
			//outputStream.write(("Total query frequency" + "\t" + ci.totalQueryFrequency + "\n\n").getBytes());
		}
		outputStream.write(("Number of correlation groups found: " + correlationGroups).getBytes());
		outputStream.close();
		System.out.println("written file: " + file.getName());
	}
	
	public static void writeCorrelations(ArrayList cra, String name) throws IOException{
		File file = new File(Config.PATHS_PROCESSED_OUTPUT, name);
		OutputStream outputStream = new BufferedOutputStream(new FileOutputStream(file));
		int correlationGroups = 0;
		
		for(int i = 0; i < cra.size(); i++){
			CorrelationInfo ci = (CorrelationInfo)cra.get(i);
			
			if(ci.correlationList.size() > 0){
				correlationGroups++;
				
				if(!"".equals(ci.query.trim())){
					outputStream.write((ci.query + "\n").getBytes());
				}
				outputStream.write(("merge segment:" + "\t" + ci.mergeSegment + "\n").getBytes());
				Collections.sort(ci.correlationList, new CorrelationValueComperator());
				
				for(int j = 0; j < ci.correlationList.size(); j++){		
					CorrelationFields cf = (CorrelationFields)ci.correlationList.get(j);
					outputStream.write((cf.correlationValue + "\t" + cf.query + "\t" + cf.frequency + "\n").getBytes());
				}
			}
			
			outputStream.write(("Number of correlating queries found" + "\t" + ci.correlationList.size() + "\n").getBytes());
			outputStream.write(("Total query frequency" + "\t" + ci.totalQueryFrequency + "\n\n").getBytes());
		}
		outputStream.write(("Number of correlation groups found: " + correlationGroups).getBytes());
		outputStream.close();
		System.out.println("written file: " + file.getName());
	}
	
	public static void writeFrequencyStatistics(HashMap correlationMap) throws IOException{
		File file = new File(Config.PATHS_PREPROCESSES_RAW_LOG_PATH, "correlation_frequency_test");
		OutputStream outputStream = new BufferedOutputStream(new FileOutputStream(file));
		
		Iterator iterator = correlationMap.entrySet().iterator();
		ArrayList sortList = new ArrayList();
		
		while(iterator.hasNext()){
			Map.Entry  hm = (Map.Entry)iterator.next();
			sortList.add(hm.getValue());
		}
		
		Collections.sort(sortList, new CorrelationTotalFrequencyComperator());
		
		for(int i = 0; i < sortList.size(); i++){
			TimeUnit tu = (TimeUnit)sortList.get(i);
			if(tu.frequency.length > 1){
				outputStream.write(("\n" + tu.totalFrequency + "\t" + tu.query + "\t" + tu.averageFrequency + "\t" + tu.standardDeviation + "\t").getBytes());
				for(int j = 0; j < tu.frequency.length; j++){
					outputStream.write((tu.frequency[j] + "\t").getBytes());
				}
			}
		}
		
		outputStream.close();
	}

}
