package ee.termgroups;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.StringTokenizer;

import ee.config.Config;
import ee.decomposition.Decompose;
import ee.decomposition.Decompose.MultiGramInfo;
import ee.decomposition.Decompose.MultiGramInfoComperator;
import ee.input.file.FileFinder;
import ee.termgroups.ReadFilteredCorrelations.SegmentFields;
import ee.termgroups.ReadFilteredCorrelations.SegmentInfo;
import ee.utilities.LevenshteinDistance;
import ee.utilities.StopWords;

public class ProcessTermGroups {
	
//	public static class ExtendedTreeNode{
//		public static int IDCounter = 0;
//		
//		public int treeNodeID;
//		public String branchName;
//		public int parentNodeID;
//		public String parentName;
//		public boolean isTopNode = false;
//			
//		public float correlationFrequency;
//		public ArrayList leafList;
//		
//		public ExtendedTreeNode(int parentID, String parentName, String branchName, float correlationFrequency){
//			this.parentNodeID = parentID;
//			this.branchName = branchName;
//			this.correlationFrequency = correlationFrequency;
//			leafList = new ArrayList();
//			ExtendedTreeNode.IDCounter++;
//			treeNodeID = ExtendedTreeNode.IDCounter;
//		}
//	}
	
	public static class TreeNode{
		public static int IDCounter = 0;
		
		public int treeNodeID;
		public int parentNodeID;
		public String parentName;
			
		public String branchName;
		public float correlationFrequency;
		public ArrayList leafList;
		public ArrayList newLeafList;
		public HashMap candidateLeafMap;
		public boolean precalculatedBranchName = false;
		
		public TreeNode(String branchName, float correlationFrequency){
			this.branchName = branchName;
			this.correlationFrequency = correlationFrequency;
			leafList = new ArrayList();
			newLeafList = new ArrayList();
			candidateLeafMap = new HashMap();
		}
		
		public TreeNode(int parentID, String parentName, String branchName, float correlationFrequency){
		this.parentNodeID = parentID;
		this.parentName = parentName;
		this.branchName = branchName;
		this.correlationFrequency = correlationFrequency;
		leafList = new ArrayList();
		newLeafList = new ArrayList();
		TreeNode.IDCounter++;
		treeNodeID = TreeNode.IDCounter;
	}
		
		public static String[] tokenizeLine(String token){
			StringTokenizer st = new StringTokenizer(token, " ");
			int tmp = st.countTokens();
			String[] out = new String[tmp];
			for(int i = 0; i < tmp; i++){
				out[i] = st.nextToken();
			}
			return out;
		}
	}
	
	public static class TreeNodeComperator implements Comparator {
		public int compare(Object sf1, Object sf2) {
			TreeNode sortInfo1 = (TreeNode)sf1;
			TreeNode sortInfo2 = (TreeNode)sf2;
			
			if (sortInfo1.correlationFrequency == sortInfo2.correlationFrequency) {
				return 0;
			} else if (sortInfo1.correlationFrequency > sortInfo2.correlationFrequency) {
				return -1;
			} else {
				return 1;
			}
		}
	}
	
	private Decompose decompose;
	private StopWords stopWords;
	private TreeNode mainGroupTermNode;
	private TreeNode allTermGroupNode;
	
	public ProcessTermGroups() throws IOException{
			decompose = new Decompose();
			stopWords = new StopWords();
	}
	
	public void processFiles(String fileNameToken) throws IOException{
		ArrayList fileList = FileFinder.getAllFiles(Config.PATHS_PROCESSED_OUTPUT, "trimmed_segment_merged_query");
		
		for(int i = 0; i < fileList.size(); i++){	
			processFile((File)fileList.get(i));
		}
	}
	
	public void processFile(File f) throws IOException{
		ArrayList groupList = ReadFilteredCorrelations.ReadInput(f);
		TreeNode mainNode = new TreeNode(f.getName(), 0);
		
		 for(int i = 0; i < groupList.size(); i++){
			SegmentInfo si = (SegmentInfo)groupList.get(i);
			TreeNode parentNode = new TreeNode(si.mergeSegment, 0);
			parentNode.precalculatedBranchName = true;
			
			for(int j = 0; j < si.queryList.size(); j++){
				SegmentFields sf = (SegmentFields)si.queryList.get(j);
				parentNode.candidateLeafMap.put(sf.query, new TreeNode(sf.query, sf.correlationValue*sf.frequency));
			}
			
			mainNode.leafList.add(parentNode);

		}	
		
		mainGroupTermNode = treeNodeCreation(mainNode);
		allTermGroupNode = secondTreeGeneration(groupList);

		ProcessTermGroups.writeTermGroups(mainGroupTermNode, "term_group_tree");
		ProcessTermGroups.writeTermGroups(allTermGroupNode, "term_group_tree_all");
		
	}
	
	private TreeNode treeNodeCreation(TreeNode parentNode){
		
		for(int i = 0; i < parentNode.leafList.size(); i++){
			TreeNode tn = (TreeNode)parentNode.leafList.get(i);
			
			while(tn.candidateLeafMap.size() > 0){
				createTreeBranch(tn);
			}
			treeNodeCreation(tn);
		}
		
		return parentNode;
	}
	
	private void createTreeBranch(TreeNode parentNode){

		String branchName;
		if(parentNode.precalculatedBranchName){
			branchName = parentNode.branchName;
		} else {
			branchName = ((MultiGramInfo)findBranchingSegment(parentNode).get(0)).multiGram;
		}
		
		ArrayList segments = decompose.simpleSegmentFinder(branchName);
		Collections.sort(segments, new MultiGramInfoComperator());
		
		if(segments.size() == 0){
			TreeNode t = ((TreeNode)((Map.Entry)parentNode.candidateLeafMap.entrySet().iterator().next()).getValue());
			segments.add(new MultiGramInfo(t.branchName, 0, 0));
		}
		
		TreeNode newNode = new TreeNode(branchName, 0);
		HashMap failedCandidateChildren = new HashMap();
		HashMap newNodeChildren = new HashMap();
		
		Iterator it = parentNode.candidateLeafMap.entrySet().iterator();
		
		while(it.hasNext()){
			Map.Entry me = (Map.Entry)it.next();
			TreeNode tn = (TreeNode)me.getValue();
			
			String tempString = new String(tn.branchName);
			
			boolean segmentMatch = false;
			
			for(int k = 0; k < segments.size(); k++){
				MultiGramInfo mgi = (MultiGramInfo)segments.get(k);
				
				if(tn.branchName.indexOf(mgi.multiGram) >= 0){
					
					tempString = tempString.replaceAll(mgi.multiGram, "");
					tempString = tempString.replaceAll("\\.", " ");
					tempString = tempString.replaceAll("  ", " ");
					tempString = tempString.trim();
			
					segmentMatch = true;
				}	
			}	
			
			tn.branchName = tempString;
			
			if(!"".equals(tempString) && tempString.length() > 1 && !stopWords.isStopWord(tempString)){
				if(segmentMatch){
					if(newNodeChildren.containsKey(tn.branchName)){
						TreeNode tno = (TreeNode)newNodeChildren.get(tn.branchName);
						tno.correlationFrequency += tn.correlationFrequency;
						newNode.correlationFrequency += tn.correlationFrequency;
					} else {
						//System.out.println(tn.branchName + " " + tn.correlationFrequency);
						newNodeChildren.put(tn.branchName, tn);
						newNode.correlationFrequency += tn.correlationFrequency;
					}
				} else {
					failedCandidateChildren.put(tn.branchName, tn);
				}
			} else {
				newNode.correlationFrequency += tn.correlationFrequency;
			}
		}
		
		parentNode.leafList.add(newNode);
		parentNode.candidateLeafMap = failedCandidateChildren;
		newNode.candidateLeafMap = newNodeChildren;
	}
	
	private ArrayList findBranchingSegment(TreeNode parentNode){
		HashMap segmentMap = new HashMap();
		
		Iterator it = parentNode.candidateLeafMap.entrySet().iterator();
		
		while(it.hasNext()){
			Map.Entry me = (Map.Entry)it.next();
			TreeNode tn = (TreeNode)me.getValue();
			
			ArrayList segmentList = decompose.simpleSegmentFinder(tn.branchName);
			for(int j = 0; j < segmentList.size(); j++){
				MultiGramInfo mgi = (MultiGramInfo)segmentList.get(j);
				
				if(segmentMap.containsKey(mgi.multiGram)){
					MultiGramInfo mg = (MultiGramInfo)segmentMap.get(mgi.multiGram);
					mg.score += mgi.score;
				} else {
					segmentMap.put(mgi.multiGram, new MultiGramInfo(mgi.multiGram, mgi.numberOfTokens, mgi.frequency));
				}
			}
		}
		
		ArrayList sortArray = new ArrayList();
		
		if(segmentMap.size() == 0){
			//System.out.println(parentNode.candidateLeafMap.entrySet().iterator().hasNext());
//			System.out.println(parentNode.branchName);
//			System.out.println(parentNode.candidateLeafMap.size());
			TreeNode t = ((TreeNode)((Map.Entry)parentNode.candidateLeafMap.entrySet().iterator().next()).getValue());
			sortArray.add(new MultiGramInfo(t.branchName, 0, 0));
			return sortArray;
		}
		
		Iterator its = segmentMap.entrySet().iterator();
		while(its.hasNext()){
			Map.Entry me = (Map.Entry)its.next();
			sortArray.add((MultiGramInfo)me.getValue());
		}
		
		Collections.sort(sortArray, new MultiGramInfoComperator());
		
		return sortArray;
	}

	private TreeNode secondTreeGeneration(ArrayList groupList) throws IOException{
		
		HashSet termGroupMap = new HashSet();
		HashMap newSegmentMap = new HashMap();
		
		TreeNode mainNode = new TreeNode("generated groups", 0);
		
		for(int i = 0; i < groupList.size(); i++){
			SegmentInfo si = (SegmentInfo)groupList.get(i);
			
			termGroupMap.add(si.mergeSegment);
		}
		
		for(int i = 0; i < groupList.size(); i++){
			SegmentInfo si = (SegmentInfo)groupList.get(i);
			
			for(int j = 0; j < si.queryList.size(); j++){
				SegmentFields sf = (SegmentFields)si.queryList.get(j);
				
				ArrayList segments = decompose.simpleSegmentFinder(sf.query);

				for(int k = 0; k < segments.size(); k++){
				//for(int k = 0; k < Math.ceil(segments.size() / 2) ; k++){
					MultiGramInfo mgi = (MultiGramInfo)segments.get(k);
					String[] tokens = TreeNode.tokenizeLine(mgi.multiGram);
					
					if(!termGroupMap.contains(mgi.multiGram) && !stopWords.isStopWord(mgi.multiGram) && mgi.multiGram.length() > 1 && tokens.length < 4){
						if(newSegmentMap.containsKey(mgi.multiGram)){
							TreeNode treeNode = (TreeNode)newSegmentMap.get(mgi.multiGram);
							treeNode.candidateLeafMap.put(sf.query, new TreeNode(sf.query, sf.correlationValue*sf.frequency));
						} else {
							TreeNode treeNode = new TreeNode(mgi.multiGram, 0);
							treeNode.candidateLeafMap.put(sf.query, new TreeNode(sf.query, sf.correlationValue*sf.frequency));
							newSegmentMap.put(mgi.multiGram, treeNode);
						}
				}
				}
				
//				sf.tokenizeQuery();
//				
//				for(int k = 0; k < sf.tokenArray.length; k++){
////					if(!termGroupMap.contains(sf.tokenArray[k]) && !stopWords.isStopWord(sf.tokenArray[k]) && sf.tokenArray[k].length() > 1){
//						if(newSegmentMap.containsKey(sf.tokenArray[k])){
//							TreeNode treeNode = (TreeNode)newSegmentMap.get(sf.tokenArray[k]);
//							treeNode.candidateLeafMap.put(sf.query, new TreeNode(sf.query, sf.correlationValue*sf.frequency));
//						} else {
//							TreeNode treeNode = new TreeNode(sf.tokenArray[k], 0);
//							treeNode.candidateLeafMap.put(sf.query, new TreeNode(sf.query, sf.correlationValue*sf.frequency));
//							newSegmentMap.put(sf.tokenArray[k], treeNode);
//						}
////					}
//				}
				
				
			}
		}
		
		System.out.println("main node leaf list size" + mainNode.leafList.size());
		
		Iterator it = newSegmentMap.entrySet().iterator();
		while(it.hasNext()){
			Map.Entry me = (Map.Entry)it.next();
			TreeNode treeNode = (TreeNode)me.getValue();
			if(treeNode.candidateLeafMap.size() > 1){
				treeNode.precalculatedBranchName = true;
				mainNode.leafList.add(treeNode);
			}
		}
		
		treeNodeCreation(mainNode);
		
		System.out.println("main node leaf list size" + mainNode.leafList.size());
		
		for(int i = 0; i < mainGroupTermNode.leafList.size(); i++){
			TreeNode tn = (TreeNode)mainGroupTermNode.leafList.get(i);
			TreeNode tno = (TreeNode)tn.leafList.get(0);
//			tno.precalculatedBranchName = true;
			mainNode.leafList.add(tn);
			//System.out.println(tno.branchName + tno.correlationFrequency);
		}
		
		System.out.println("main node leaf list size" + mainNode.leafList.size());
//		
//		for (TreeNode tn : (ArrayList<TreeNode>)mainNode.leafList) {
//			System.out.println(tn.branchName + tn.correlationFrequency);
//		}
		
		return mainNode;
	}
	
	public HashMap getMainTermGroups(){
		return createGroupHash(mainGroupTermNode);
	}
	
	public HashMap getAllTermGroups(){
		return createGroupHash(allTermGroupNode);
	}
	
	private HashMap createGroupHash(TreeNode mainNode){
		int maxCount = 5;
		
		ArrayList filteredTermGroups = new ArrayList();
		
		for(int i = 0; i < mainNode.leafList.size(); i++){
			
			ArrayList groupList = new ArrayList();
			TreeNode tn = (TreeNode)mainNode.leafList.get(i);
			
			readTreeNodeFilteredTermGroups(tn, groupList);
			filteredTermGroups.add(groupList);
		}
		
		HashMap outputHash = new HashMap();
		
		for(int i = 0; i < filteredTermGroups.size(); i++){
			ArrayList groupList = (ArrayList)filteredTermGroups.get(i);
			if(groupList.size() == 0){
				continue;
			}
			String groupName = ((TreeNode)groupList.get(0)).branchName;
			Collections.sort(groupList, new TreeNodeComperator());
			
			ArrayList outGroupList = new ArrayList();

			for(int j = 0; j < Math.min(groupList.size(), maxCount); j++){
				TreeNode tn = (TreeNode)groupList.get(j);
				outGroupList.add(tn);
				//System.out.println(tn.branchName + "\t" + tn.correlationFrequency);
			}
			
			ArrayList groupNameSegments = decompose.simpleSegmentFinder(groupName);
			for(int j = 0; j < groupNameSegments.size(); j++){
				MultiGramInfo mgi = (MultiGramInfo)groupNameSegments.get(j);
				
				if(!outputHash.containsKey(mgi.multiGram)){
					ArrayList grpList = new ArrayList();
					grpList.add(outGroupList);
					outputHash.put(mgi.multiGram, grpList);
				} else {
					ArrayList grpList = (ArrayList)outputHash.get(mgi.multiGram);
					grpList.add(outGroupList);
					//outputHash.put(mgi.multiGram, mergeArrayLists(one, outGroupList, maxCount));
				}
			}
		}
		
		return outputHash;
	}
	
	public static ArrayList mergeArrayLists(ArrayList one, ArrayList two, int maxCount){
		if(one == two){
			System.out.println(one.size());
			return one;
		}
		
		ArrayList outputList = new ArrayList();
		
		for(int i = 0; i < two.size(); i++){
			one.add((TreeNode)two.get(i));
		}
		
		Collections.sort(one, new TreeNodeComperator());
		
		for(int i = 0; i < Math.min(one.size(), maxCount); i++){
			outputList.add((TreeNode)one.get(i));
		}
		
		return outputList;
	}
	
	private void readTreeNodeFilteredTermGroups(TreeNode tn, ArrayList termList){
		
		for(int i = 0; i < tn.leafList.size(); i++){
			TreeNode tno = (TreeNode)tn.leafList.get(i);
			
			if(!tn.precalculatedBranchName){
				tno.branchName = tn.branchName + " " + tno.branchName;
			} 
			termList.add(tno);
			
			readTreeNodeFilteredTermGroups(tno, termList);
			
		}
		tn.precalculatedBranchName = true;
	}
	
	public HashMap getMainTermHashID(){
		return createTreeHash(mainGroupTermNode);
	}
	
	public HashMap getAllTermHashID(){
		return createTreeHash(allTermGroupNode);
	}
	
	private HashMap createTreeHash(TreeNode mainNode){
		
		HashMap treeGroupHash = new HashMap();
		TreeNode newMainNode = new TreeNode(0, "main", mainNode.branchName, mainNode.correlationFrequency);
		newMainNode.leafList = mainNode.leafList;
		
		for(int i = 0; i < newMainNode.leafList.size(); i++){
			
			TreeNode tn = (TreeNode)newMainNode.leafList.get(i);
			TreeNode etn = new TreeNode(newMainNode.parentNodeID, newMainNode.branchName, tn.branchName, tn.correlationFrequency);
			etn.leafList = tn.leafList;
			tn.newLeafList.add(etn);
			
			//System.out.println(etn.treeNodeID + etn.branchName + etn.isTopNode);
			
			readTreeNodesForTreeHashCreation(etn, treeGroupHash, true);
			//treeGroupHash.put(etn.treeNodeID, etn);
		}
		
		//System.out.println("tree group hash size" + treeGroupHash.size());
		
		return treeGroupHash;
	}
	
	private void readTreeNodesForTreeHashCreation(TreeNode et, HashMap termHash, boolean isFirst){
		
		for(int i = 0; i < et.leafList.size(); i++){
			TreeNode tn = (TreeNode)et.leafList.get(i);
			
			TreeNode etn = new TreeNode(et.treeNodeID, et.branchName, tn.branchName, tn.correlationFrequency);
			etn.leafList = tn.leafList;
			et.newLeafList.add(etn);
			
			//System.out.println(etn.treeNodeID + etn.branchName + etn.precalculatedBranchName);
			
			if(isFirst){
				etn.precalculatedBranchName = true;
			}
			
			termHash.put(new String(""+etn.treeNodeID), etn);

			readTreeNodesForTreeHashCreation(etn, termHash, false);
			
		}
	}
	
	public HashMap getReverseTreeHash(HashMap treeMap){
		HashMap reverseTreeHash = new HashMap();
		
		Iterator it = treeMap.entrySet().iterator();
		while(it.hasNext()){
			Map.Entry me = (Map.Entry)it.next();
			TreeNode etn = (TreeNode)me.getValue();
			if(etn.precalculatedBranchName){
				reverseTreeHash.put(etn.branchName, etn);
			}
		}
		
		return reverseTreeHash;
	}
	
	public static void writeTermGroups(TreeNode topNode, String name) throws IOException{
		File f = new File(Config.PATHS_PROCESSED_OUTPUT, name);
		OutputStream outputStream = new BufferedOutputStream(new FileOutputStream(f));
		
		Collections.sort(topNode.leafList, new TreeNodeComperator());
		
		for(int i = 0; i < topNode.leafList.size(); i++){
			writeTermGroupNode((TreeNode)topNode.leafList.get(i), "", outputStream);
		}
		
		outputStream.close();
		System.out.println("written file: " + f.getName());
	}
	
	public static void writeTermGroupNode(TreeNode treeNode, String buffer, OutputStream outputStream) throws IOException{
		buffer = buffer + "  ";
		if(treeNode.correlationFrequency != 0.0f){
			outputStream.write((buffer + treeNode.branchName + "\t" + treeNode.correlationFrequency + "\n").getBytes());
		}
			
		for(int i = 0; i < treeNode.leafList.size(); i++){
			TreeNode tn = (TreeNode)treeNode.leafList.get(i);
			writeTermGroupNode(tn, buffer, outputStream);
		}
	}
	
	public Decompose getDecompose(){
		return decompose;
	}
	
	public void testLevensthein(){
		System.out.println(LevenshteinDistance.computeLevenshteinDistance("britney".toCharArray(), "britany".toCharArray()));
		System.out.println(LevenshteinDistance.computeLevenshteinDistance("britney".toCharArray(), "brittany".toCharArray()));
		System.out.println(LevenshteinDistance.computeLevenshteinDistance("britney".toCharArray(), "brittney".toCharArray()));
		
//		Group name: britney spears
//		new	9
//		pictures bald	52
//		britany	7
//		news	4
//		brittney	5
//		pictures	135
//		bald	65
//		brittany	12
//		bald pictures	12
	}
	

	public static void main(String args[]) throws IOException{
		ProcessTermGroups ptg = new ProcessTermGroups();
		ptg.processFile(new File(Config.PATHS_PROCESSED_OUTPUT, "trimmed_segment_merged_query_filtered_correlations70-5-0-6-25-3"));
		ptg.getAllTermHashID();
	}

}
