package ee.termgroups;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.StringTokenizer;

import ee.input.file.ReadFile;

public class ReadFilteredCorrelations {
	
	public static class SegmentFields{
		String[] tokenArray;
		String query;
		int frequency;	
		float correlationValue;
		public SegmentFields(String q, int f, float c){
			query = q;
			frequency = f;
			correlationValue = c;
			tokenizeQuery();
		}
		
		public void tokenizeQuery(){
			StringTokenizer st = new StringTokenizer(query, " ");
			tokenArray = new String[st.countTokens()];
			for(int i = 0; i < tokenArray.length; i++){
				tokenArray[i] = st.nextToken();
			}
		}
	}
	
	public static class SegmentInfo{
		String mergeSegment;
		ArrayList queryList;
		int totalQueryFrequency;
		int maxNumberOfTerms;
		
		public SegmentInfo(String mergeSegment){
			this.mergeSegment = mergeSegment;
			queryList = new ArrayList();
		}
	}
	
	public static ArrayList ReadInput(File file) throws IOException{
		
		ReadFile rf = new ReadFile(file);
		ArrayList correlationInfoList = new ArrayList();
		
		SegmentInfo si = null;
		
		String tempString = "";
		while((tempString = rf.getNextLine()) != null){
			
			if("".equals(tempString.trim())){
				correlationInfoList.add(si);
				si = null;
				continue;
			}
			
			StringTokenizer st = new StringTokenizer(tempString, "\t");
			
			if(st.countTokens() == 2 && si == null){
				st.nextToken();
				si = new SegmentInfo(st.nextToken());
			} else if(st.countTokens() == 3){
				float correlationValue = Float.parseFloat(st.nextToken());
				String query = st.nextToken();
				int frequency = Integer.parseInt(st.nextToken());
				SegmentFields sf = new SegmentFields(query, frequency, correlationValue);
				si.queryList.add(sf);
				si.totalQueryFrequency += frequency;
				if(si.maxNumberOfTerms < sf.tokenArray.length){
					si.maxNumberOfTerms = sf.tokenArray.length;
				}
			}
		}
		return correlationInfoList;
	}
}
