package ee.termgroups;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import ee.config.Config;
import ee.decomposition.Decompose;
import ee.decomposition.Decompose.MultiGramInfo;
import ee.decomposition.Decompose.MultiGramInfoComperator;
import ee.temporalcorrelation.TemporalCorrelationPreprocessing.CorrelationFields;
import ee.temporalcorrelation.TemporalCorrelationPreprocessing.CorrelationInfo;

public class SimpleTermGroups {
	
	public static class ExtractionInfo{
		String groupSegment;
		HashMap extractedTermsMap;
		
		public ExtractionInfo(String groupSegment, HashMap extractedTermsMap){
			this.groupSegment = groupSegment;
			this.extractedTermsMap = extractedTermsMap;
		}
	}
	
	public static ArrayList createGroupsWithSegments(ArrayList filteredCorrelationResultArray, Decompose decompose) throws IOException{
		
		File file = new File(Config.PATHS_PROCESSED_OUTPUT, "term_groups_with_segments");
		OutputStream outputStream = new BufferedOutputStream(new FileOutputStream(file));
		
		for(int i = 0; i < filteredCorrelationResultArray.size(); i++){
			CorrelationInfo ci = (CorrelationInfo)filteredCorrelationResultArray.get(i);
			
			HashMap segmentCandidateHash = new HashMap();
			
			outputStream.write(("\nGroup title: " + ci.mergeSegment + "\n").getBytes());
			
			for(int j = 0; j < ci.correlationList.size(); j++){
				CorrelationFields cf = (CorrelationFields)ci.correlationList.get(j);
				
				ArrayList segments = decompose.simpleSegmentFinder(cf.query);
				
				for(int k = 0; k < segments.size(); k++){
					MultiGramInfo mgi = (MultiGramInfo)segments.get(k);
					if(segmentCandidateHash.containsKey(mgi.multiGram)){
						MultiGramInfo mgiold = (MultiGramInfo)segmentCandidateHash.get(mgi.multiGram);
						mgiold.frequency += mgi.frequency;
						mgiold.score += mgi.score;
					} else {
						segmentCandidateHash.put(mgi.multiGram, mgi);
					}
				}
			}
			
			segmentCandidateHash.remove(ci.mergeSegment);
			
			ArrayList sortArray = new ArrayList();
			Iterator it = segmentCandidateHash.entrySet().iterator();
			while(it.hasNext()){
				Map.Entry me = (Map.Entry)it.next();
				sortArray.add((MultiGramInfo)me.getValue());				
			}
			Collections.sort(sortArray, new MultiGramInfoComperator());
			
			int range = Math.min(5, sortArray.size());
			
			for(int j = 0; j < range; j++){
				MultiGramInfo mgi = (MultiGramInfo)sortArray.get(j);
				outputStream.write((mgi.multiGram + "\n").getBytes());
			}
		}
		
		outputStream.close();
		System.out.println("written file: " + file.getName());
		return null;
	}
	
	public static void createGroupsByExtraction(ArrayList filteredCorrelationResultArray, Decompose decompose) throws IOException{
		
		File file = new File(Config.PATHS_PROCESSED_OUTPUT, "term_groups_with_extraction");
		OutputStream outputStream = new BufferedOutputStream(new FileOutputStream(file));
		ArrayList results = new ArrayList();
		Pattern p = Pattern.compile("  ");
		
		for(int i = 0; i < filteredCorrelationResultArray.size(); i++){
			CorrelationInfo ci = (CorrelationInfo)filteredCorrelationResultArray.get(i);
			
			ArrayList segmentList = decompose.simpleSegmentFinder(ci.mergeSegment);
			HashMap remainingQuery = new HashMap();
			
			for(int j = 0; j < ci.correlationList.size(); j++){
				CorrelationFields cf = (CorrelationFields)ci.correlationList.get(j);
				for(int k = 0; k < segmentList.size(); k++){
					MultiGramInfo mgi = (MultiGramInfo)segmentList.get(k);
					cf.query = cf.query.replaceFirst(mgi.multiGram, "");
				}
				cf.query = cf.query.trim();
				if("".equals(cf.query)){
					continue;
				}
				Matcher m = p.matcher(cf.query);
				cf.query = m.replaceAll(" ");
				if(remainingQuery.containsKey(cf.query)){
					remainingQuery.put(cf.query, new Integer(((Integer)remainingQuery.get(cf.query)).intValue() + cf.frequency));					
				} else {
					remainingQuery.put(cf.query, new Integer(cf.frequency));
				}
			}
			
			results.add(new ExtractionInfo(ci.mergeSegment, remainingQuery));
		}
		
		for(int i = 0; i < results.size(); i++){
			ExtractionInfo group = (ExtractionInfo)results.get(i);
			outputStream.write(("\nGroup name: " + group.groupSegment + "\n").getBytes());
			
			Iterator it = group.extractedTermsMap.entrySet().iterator();
			while(it.hasNext()){
				Map.Entry me = (Map.Entry)it.next();
				String terms = (String)me.getKey();
				int frequency = ((Integer)me.getValue()).intValue();
				
				outputStream.write((terms + "\t" + frequency + "\n").getBytes());
			}
		}
		outputStream.close();
		System.out.println("written file: " + file.getName());
	}
}
