package ee.web;

import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.net.URL;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import ee.utilities.Pair;

public class Query {
	private static final int HITS_PER_PAGE = 10;
	private static final int MIN_HITS = 10;
	private static final int MAX_PAGES = 10;
	private static final Object UNIQUE_FIELD = "uri";
	private static final Object ABSTRACT_FIELD = "abstract";
	private static final Object TITLE_FIELD = "title";
	
	private int hitCount;
	private List<Map<String, String>> hits = new ArrayList<Map<String, String>>();
	private Set<Pair<String, String>> abstractAndTitles = new HashSet<Pair<String, String>>();
	private Set<String> uris = new HashSet<String>();
	private String language;
	private String query;
	private String canonicalQuery;
	public Query(String query) {
		this.query = query;
	}

	private boolean fetchPage(int page) throws IOException {
		StringBuilder queryString = new StringBuilder();
		if (language != null) {
			queryString.append("language:");
			queryString.append(language);
		}
		queryString.append(' ');
		queryString.append(query);
		canonicalQuery = queryString.toString();

		int offset = page * HITS_PER_PAGE;

		URL url = new URL("http", "localhost", 5810, "/?query=" + URLEncoder.encode(canonicalQuery, "UTF-8") + "&offset=" + offset);

		InputStream in = url.openStream();

		final ArrayList<Map<String, String>> pageHits = new ArrayList<Map<String, String>>();

		SAXParserFactory factory = SAXParserFactory.newInstance();
		SAXParser parser;
		try {
			parser = factory.newSAXParser();

			parser.parse(in, new DefaultHandler() {
				private String fieldName;

				private Map<String, String> hit;

				private StringBuilder value;

				@Override
				public void characters(char[] ch, int start, int length)
						throws SAXException {
					if (value != null) {
						value.append(ch, start, length);
					}
				}

				@Override
				public void startElement(String uri, String localName,
						String qName, Attributes attributes)
						throws SAXException {
					if ("field".equals(qName)) {
						fieldName = attributes.getValue("name");
						value = new StringBuilder();
					} else if ("hit".equals(qName)) {
						hit = new HashMap<String, String>();
					} else if ("result".equals(qName)) {
						try {
							hitCount = Integer.parseInt(attributes
									.getValue("total-hit-count"));
						} catch (NumberFormatException e) {
							hitCount = 0;
						}
					}
				}

				@Override
				public void endElement(String uri, String localName,
						String qName) throws SAXException {
					if ("hit".equals(qName)) {
						pageHits.add(hit);
						hit = null;
					}

					if ("field".equals(qName)) {
						hit.put(fieldName, value.toString());
						value = null;
					}
				}
			});
		} catch (ParserConfigurationException e) {
			IOException ioe = new IOException(e);
			throw ioe;
		} catch (SAXException e) {
			IOException ioe = new IOException(e);
			throw ioe;
		}

		for (Map<String, String> hit : pageHits) {
			if (hits.size() >= MIN_HITS) {
				break;
			}

			String id = hit.get(UNIQUE_FIELD);
			if (id == null) {
				continue;
			}

			Pair<String, String> abstractTitle = Pair.make(hit.get(ABSTRACT_FIELD), hit.get(TITLE_FIELD));

			if (!uris.contains(id)&& !abstractAndTitles.contains(abstractTitle)) {
				hits.add(hit);
				abstractAndTitles.add(abstractTitle);
				uris.add(id);
			} else {
				//System.out.println(id);
			}
		}

		return pageHits.size() == HITS_PER_PAGE;

	}

	public void execute() throws IOException {
		int page = 0;
		while (hits.size() < MIN_HITS && page < MAX_PAGES && fetchPage(page)) {
			++page;
		}
	}

	public int getHitCount() {
		return hitCount;
	}

	public List<Map<String, String>> getHits() {
		return hits;
	}

	public String getQuery() {
		return query;
	}

	public void print() throws UnsupportedEncodingException {
		System.out.printf("%s (%s)\n", new Object[] { query,
				URLEncoder.encode(query, "UTF-8") });

		System.out.println("Hits: " + hits.size());
		for (Map<String, String> hit : hits) {
			System.out.printf("%5s \"%s\" %s\n", new Object[] {
					hit.get("relevancy"), hit.get("title"), hit.get("uri") });
		}
		System.out.println();
	}

	public boolean isDone() {
		return hits != null;
	}

	public void setLanguage(String language) {
		this.language = language;
	}
}