package ee.web;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.HashMap;

import org.mortbay.jetty.Server;

import ee.config.Config;
import ee.decomposition.Decompose;
import ee.evaluation.Evaluation;
import ee.termgroups.ProcessTermGroups;

public class QueryServer {
	private Server server;
	private HashMap mainTermGroupHash;
	private HashMap allTermGroupHash;
	private Decompose decompose;
	
	private HashMap mainTreeHash;
	private HashMap allTreeHash;
	private HashMap reverseMainTreeHash;
	private HashMap reverseAllTreeHash;
	
	private Evaluation evaluation;
	
	public QueryServer() throws Exception {
		ProcessTermGroups ptg = new ProcessTermGroups();
		ptg.processFile(new File(Config.PATHS_PROCESSED_OUTPUT, "trimmed_segment_merged_query_filtered_correlations70-5-0-6-25-3"));
		mainTermGroupHash = ptg.getMainTermGroups();
		allTermGroupHash = ptg.getAllTermGroups();
		decompose = ptg.getDecompose();
		
		mainTreeHash = ptg.getMainTermHashID();
		allTreeHash = ptg.getAllTermHashID();
		reverseMainTreeHash = ptg.getReverseTreeHash(mainTreeHash);
		reverseAllTreeHash = ptg.getReverseTreeHash(allTreeHash);
		
		evaluation = new Evaluation();
		
		init();
		run();
	}

	private void init() throws FileNotFoundException, IOException, ClassNotFoundException {
		server = new Server(12022);
		server.addHandler(new SimpleQueryHandler(decompose, mainTermGroupHash, allTermGroupHash, mainTreeHash, allTreeHash, reverseMainTreeHash, reverseAllTreeHash, evaluation));
	}

	private void run() throws Exception {
		server.start();
		server.join();
	}
	
	public static void main(String[] args) throws Exception {
		new QueryServer();
	}
}

