package ee.web;

import java.io.IOException;
import java.io.PrintWriter;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.mortbay.jetty.Request;
import org.mortbay.jetty.handler.DefaultHandler;

import ee.decomposition.Decompose;
import ee.decomposition.Decompose.MultiGramInfo;
import ee.decomposition.Decompose.MultiGramInfoComperator;
import ee.evaluation.Evaluation;
import ee.evaluation.Evaluation.EvaluationInfo;
import ee.termgroups.ProcessTermGroups;
import ee.termgroups.ProcessTermGroups.TreeNode;

public class SimpleQueryHandler extends DefaultHandler {	
	private Decompose decompose;
	private HashMap mainTermGroupHash;
	private HashMap allTermGroupHash;
	
	private HashMap mainTreeHash;
	private HashMap allTreeHash;
	private HashMap reverseMainTreeHash;
	private HashMap reverseAllTreeHash;
	
	private int improvedSearchCount;
	
	private Evaluation evaluation;
	
	public SimpleQueryHandler(Decompose decompose, HashMap mainTermGroupHash, HashMap allTermGroupHash, HashMap mainTreeHash, HashMap allTreeHash, HashMap reverseMainTreeHash, HashMap reverseAllTreeHash, Evaluation evaluation){
		this.decompose = decompose;
		this.mainTermGroupHash = mainTermGroupHash;
		this.allTermGroupHash = allTermGroupHash;
		
		this.mainTreeHash = mainTreeHash;
		this.allTreeHash = allTreeHash;
		this.reverseMainTreeHash = reverseMainTreeHash;
		this.reverseAllTreeHash = reverseAllTreeHash;
		this.evaluation = evaluation;
	}
	
	public void handle(String uri, HttpServletRequest request, HttpServletResponse response, int status) throws IOException, ServletException {
		String query = request.getParameter("query");
		if (query == null) {
			query = "";
		} else {
			query = query.trim();
		}
		
		System.out.println(query);

		PrintWriter writer = response.getWriter(); 
		writer.print(HTMLCode.HTML_TOP);
		String mode = request.getParameter("mode");
		String evaluation = request.getParameter("evaluation");
		String cache = request.getParameter("cache");
		String treeNavigation = request.getParameter("navigation");
		String results = request.getParameter("results");
		
		String groupChosen = request.getParameter("group_chosen");
		String treeNavigationNodeID = request.getParameter("tree_navigation_node_id");
		String resultsForUserName = request.getParameter("results_for_user_name");
		
		QueryFactory qf = new QueryFactory();
		qf.setLanguage("english");

		if(cache != null){
			
			Query qu = qf.createQuery(query);
			qu.execute();

			for (Map<String, String> hit : qu.getHits()) {
				writer.print(hit.get("body"));
			}
		} else if(resultsForUserName != null){
			writerResultsForUserName(writer, resultsForUserName);
		} else if (!"".equals(query)){
			writer.print(HTMLCode.searchForm(query, mode, evaluation, treeNavigation));
			
			Query q = qf.createQuery(query);
			q.execute();
			
			if("true".equals(evaluation)){
				HTMLCode.evaluationID = 0;
				writer.append("<form id=\"evaluation\" name=\"evaluation\" method=\"get\" action=\"\">");
			}
			
			simpleRegularOutput(q, writer, "");
			
			HTMLCode.evaluationID = 0;
		} else if (results != null){
			writer.print(HTMLCode.searchForm(query, mode, evaluation, treeNavigation));
			submittedResults(request, writer, query);
		} else {
			writer.print(HTMLCode.searchForm(query, mode, evaluation, treeNavigation));
		}
		
		if(treeNavigationNodeID != null){	
			HashMap treeGroupHash;
			if("improved".equals(mode)){
				treeGroupHash = mainTreeHash;
			} else {
				treeGroupHash = allTreeHash;
			}
			
			TreeNode etn = (TreeNode)treeGroupHash.get(treeNavigationNodeID);

			if(etn != null){
				writeTreeNode(writer, etn, qf, treeGroupHash, mode, evaluation, treeNavigation, query);
			} else {
				writer.append("tree navigation node id " + treeNavigationNodeID + " not found");
			}
		} else if (!"".equals(query)){
			if(mode != null){
				if("regular".equals(mode)){		
					regularQuery(qf, query, evaluation, writer);
				} else {
					HashMap termGroupHash;
					if("improved".equals(mode)){
						termGroupHash = mainTermGroupHash;
					} else {
						termGroupHash = allTermGroupHash;
					}
					
					if(groupChosen != null){
						String segment = query;
						
						if(termGroupHash.containsKey(segment)){
							
							ArrayList candidateArray = (ArrayList)termGroupHash.get(segment);					
							ArrayList termList = new ArrayList();
							
							for(int i = 0; i < candidateArray.size(); i++){
								termList.add(candidateArray.get(i));
							}
							
							while(termList.size() > 1){
								termList.add(ProcessTermGroups.mergeArrayLists((ArrayList)termList.remove(0), (ArrayList)termList.remove(0), 5));
							}
							
							termList = (ArrayList)termList.get(0);
							
							simpleGroupOutput(termList, evaluation, qf, writer);
							
						} 
//						else {
//							regularQuery(qf, query, evaluation, writer);
//						}
					} else {
						
						ArrayList segments = decompose.simpleSegmentFinder(query);
						Collections.sort(segments, new MultiGramInfoComperator());
						
						HashMap<String, ArrayList> groupHash = new HashMap<String, ArrayList>();
						
						for(int i = 0; i < segments.size(); i++){
							MultiGramInfo mgi = (MultiGramInfo)segments.get(i);
							if(termGroupHash.containsKey(mgi.multiGram)){
								ArrayList list = (ArrayList)termGroupHash.get(mgi.multiGram);
								for(int j = 0; j < list.size(); j++){
									ArrayList termList = (ArrayList)list.get(j);
									TreeNode tn = (TreeNode)termList.get(0);
									groupHash.put(tn.branchName, list);
								}
							}
						}
						
						if(groupHash.size() == 1){
							String name = (String)((Map.Entry)groupHash.entrySet().iterator().next()).getKey();
							
//							ArrayList arrayTermList = (ArrayList)((Map.Entry)groupHash.entrySet().iterator().next()).getValue();
//							ArrayList termList = (ArrayList)arrayTermList.get(0);
							
							if(treeNavigation != null){
								
								HashMap reverseTree;
								if("improved".equals(mode)){
									reverseTree = reverseMainTreeHash;
								} else {
									reverseTree = reverseAllTreeHash;
								}
								
								//TreeNode tn = (TreeNode)termList.get(0);
								
								TreeNode outNode = (TreeNode)reverseTree.get(name);
								
								if(outNode != null){
									writeTreeNode(writer, outNode, qf, termGroupHash, mode, evaluation, treeNavigation, query);
								}
								
								
							} else {
								ArrayList candidateArray = (ArrayList)((Map.Entry)groupHash.entrySet().iterator().next()).getValue();
								
								ArrayList termList = new ArrayList();
								
								for(int i = 0; i < candidateArray.size(); i++){
									termList.add(candidateArray.get(i));
								}
								
								while(termList.size() > 1){
									termList.add(ProcessTermGroups.mergeArrayLists((ArrayList)termList.remove(0), (ArrayList)termList.remove(0), 5));
								}
								
								termList = (ArrayList)termList.get(0);
								
								termList = (ArrayList)candidateArray.get(0);
								
								simpleGroupOutput(termList, evaluation, qf, writer);
							}

						} else if (groupHash.size() > 1){
							
							writer.append(HTMLCode.startBlock(1000, "Different groups found:", 2));
							
							Iterator it = groupHash.entrySet().iterator();
							
							while(it.hasNext()){
								Map.Entry me = (Map.Entry)it.next();
								
								String name = (String)me.getKey();
								
								//ArrayList termList = (ArrayList)me.getValue();
								
								//TreeNode mainNode = (TreeNode)termList.get(0);
								
								if(treeNavigation != null){
									
//									System.out.println(treeNavigation);
//									System.out.println(name);
									
									HashMap reverseID;
									if("improved".equals(mode)){
										reverseID = reverseMainTreeHash;
									} else {
										reverseID = reverseAllTreeHash;
									}
									
									TreeNode tn = (TreeNode)reverseID.get(name);
									
									if(tn != null){
										writer.append("<a href=\"?query="+query+"&mode="+mode+"&evaluation="+evaluation+"&navigation="+treeNavigation+"&tree_navigation_node_id="+tn.treeNodeID+"\">"+name+"</a><br>");
									} else {
										writer.append("<a href=\"/?query="+name+"&mode="+mode+"&evaluation="+evaluation+"&navigation="+treeNavigation+"&group_chosen=true\">");
										writer.append(name + "</a><br>");
									}
									
								} else {
									writer.append("<a href=\"/?query="+name+"&mode="+mode+"&evaluation="+evaluation+"&group_chosen=true\">");
									writer.append(name + "</a><br>");
								}
							}
							
						} 
//						else {
//							regularQuery(qf, query, evaluation, writer);
//						}
					}
				}
			}
		} 
//		else {
//			writer.print(HTMLCode.searchForm(query, mode, evaluation, treeNavigation));
//		}
		
		improvedSearchCount = HTMLCode.evaluationID;
		writer.append("</td></tr>");
		if("true".equals(evaluation) && !"".equals(query)){
			String queryNumber = request.getParameter("query_number");
			int qn = 20;
			if(queryNumber != null){
				try {
					qn = Integer.parseInt(queryNumber.trim());
				} catch (NumberFormatException e) {
					qn = 0;
				}
			}
			writer.append(HTMLCode.evaluationSubmit(qn));
			writer.append("<input name=\"mode\" type=\"hidden\" id=\"mode\" value=\""+mode+"\" />");
			writer.append("<input name=\"evaluation\" type=\"hidden\" id=\"evaluation\" value=\""+evaluation+"\" />");
			writer.append("<input name=\"navigation\" type=\"hidden\" id=\"navigation\" value=\""+treeNavigation+"\" />");
			writer.append("<input name=\"improved_result_count\" type=\"hidden\" id=\"improved_result_count\" value=\""+improvedSearchCount+"\" />");
			writer.append("</form>");
		}
		writer.append("</table>");
		writer.print(HTMLCode.HTML_BOTTOM);

		response.setStatus(200);
		((Request) request).setHandled(true);
	}

	private void writeTreeNode(PrintWriter writer, TreeNode etn, QueryFactory qf, HashMap treeGroupHash, String mode, String evaluation, String treeNavigation, String query) throws IOException{
		treeNodeNavigation(writer, etn, mode, evaluation, treeNavigation, query);
		
		int freq = 3;
		if(etn.leafList.size() > 3){
			freq = 2;
		} else if(etn.leafList.size() == 0){
			freq = 10;
		}
		
		Query q = qf.createQuery(etn.branchName);
		q.execute();
		
		if(treeGroupHash.containsKey(""+etn.parentNodeID)){
			writer.append(HTMLCode.startBlock("Back", mode, ""+etn.parentNodeID, 2, query, evaluation));
			//regularOutput(writer, q, freq, false, evaluation);
			writer.append(HTMLCode.endBlock()+"<br>");
		}
		
		writer.append(HTMLCode.startBlock(etn.branchName, mode, ""+etn.treeNodeID, 2, query, evaluation));
		regularOutput(writer, q, freq+1, false, evaluation);
		writer.append(HTMLCode.endBlock());
		
		for(int i = 0; i < etn.newLeafList.size(); i++){
			TreeNode tn = (TreeNode)etn.newLeafList.get(i);
			
			q = qf.createQuery(tn.branchName);
			q.execute();
			
			if(q.getHitCount() > 0){
				writer.append(HTMLCode.startBlock(tn.branchName.replaceAll(etn.branchName, "").trim(), mode, ""+tn.treeNodeID, 3, query, evaluation));		
				regularOutput(writer, q, freq, false, evaluation);
				writer.append(HTMLCode.endBlock());
			}
		}
	}
	
	private void regularQuery(QueryFactory qf, String query, String evaluation, PrintWriter writer) throws IOException{
		Query qu = qf.createQuery(query);
		qu.execute();

		if(qu.getHitCount() > 0){
			writer.append(HTMLCode.startBlock(1000, query, 2));
			regularOutput(writer, qu, 10, true, evaluation);
			writer.append(HTMLCode.endBlock());
		}
	}
	
	private void simpleGroupOutput(ArrayList termList, String evaluation, QueryFactory qf, PrintWriter writer) throws IOException{
		
		for(int i = 0; i < termList.size(); i++){
			TreeNode tn = (TreeNode)termList.get(i);
			
			Query q = qf.createQuery(tn.branchName);
			q.execute();
			
			writer.append(HTMLCode.startBlock(1000, tn.branchName, 2));		
			regularOutput(writer, q, 2, false, evaluation);
			writer.append(HTMLCode.endBlock());
			
		}
	}
	
	private void simpleRegularOutput(Query q, PrintWriter writer, String evaluation) throws UnsupportedEncodingException{
		
		writer.append("<table width=\"1100\" border=\"0\" align=\"center\" cellpadding=\"0\" cellspacing=\"0\"><tr><td width=\"550\" valign=\"top\">");
		writer.append("<br><br><div class=\"title2\">Regular Search Results</div><br><br>");
		
		regularOutput(writer, q, 10, false, evaluation);
		writer.append("</td><td width=\"550\" valign=\"top\">");
		
	}
	
	private void regularOutput(PrintWriter writer, Query q, int maxNumber, boolean debug, String evaluation) throws UnsupportedEncodingException{
		int counter = 0;
		
		for (Map<String, String> hit : q.getHits()) {
			counter++;
			
			if(counter > maxNumber || hit == null){
				break;
			}
			
			writer.append(" <table width=\"100%\" cellspacing=\"0\"><tr><td bgcolor=\"#DFDFDF\"><div align=\"center\" class=\"small\">");
			writer.append(hit.get("title"));
			writer.append("</div></td></tr><tr><td><div align=\"center\"><span  class=\"smaller\">");
			writer.append(hit.get("abstract"));
			writer.append("</div></td></tr><tr><td><div align=\"center\" class=\"smaller\"><a href=\"");
			writer.append(hit.get("uri"));
			writer.append("\" target=\"_blank\">url</a>&nbsp;&nbsp;");
			writer.append("<a href=\"/?query=uri:"+URLEncoder.encode(hit.get("uri"), "UTF-8")+"&cache=true\" target=\"_blank\">cache</a></span></div></td></tr>");
			writer.append("");
			
			writer.append(HTMLCode.evaluationButtons(evaluation) +"</table>");

			if(debug){
				writer.println("<!--");
				for (Map.Entry<String, String> field : hit.entrySet()) {
					String value = field.getValue();
	
					writer.println(field.getKey()
							+ " = "
							+ value.substring(0, Math.min(value
									.length(), 150)));
				}
				writer.println("-->");
			}
		}
	}
	
	private void treeNodeNavigation(PrintWriter writer, TreeNode treeNode, String mode, String evaluation, String treeNavigation, String query){
		//System.out.println(treeNode.branchName);
		
		HashMap treeGroupHash;
		if("improved".equals(mode)){
			treeGroupHash = mainTreeHash;
		} else {
			treeGroupHash = allTreeHash;
		}
		
		TreeNode parentNode = treeNode;
		
		while(treeGroupHash.containsKey(""+parentNode.parentNodeID)){
			parentNode = (TreeNode)treeGroupHash.get(""+parentNode.parentNodeID);
			//System.out.println(parentNode.branchName + parentNode.treeNodeID);
		}
		
		writer.append("<br>"+HTMLCode.startBlock(400, parentNode.branchName, 3));
		//writer.append("<div align=\"center\">");
		writeTreeNodeNavigationNode(writer, parentNode, mode, evaluation, treeNavigation, parentNode.branchName, query);
		//writer.append("</div>");
		
		writer.append(HTMLCode.endBlock()+"<br>");
		
		if("true".equals(evaluation)){
			writer.append(HTMLCode.startBlock(400, "Does the tree structure give:", 3));
			writer.append(HTMLCode.treeEvaluation());
			writer.append(HTMLCode.endBlock()+"<br>");
		}
	}
	
	private void writeTreeNodeNavigationNode(PrintWriter writer, TreeNode treeNode, String mode, String evaluation, String treeNavigation, String nodeName, String query){
		
		writer.append("<a href=\"?query="+query+"&mode="+mode+"&evaluation="+evaluation+"&navigation="+treeNavigation+"&tree_navigation_node_id="+treeNode.treeNodeID+"\">"+nodeName+"</a><br>");
		
		for(int i = 0; i < treeNode.newLeafList.size(); i++){
			TreeNode leaf = (TreeNode)treeNode.newLeafList.get(i);
			String text = leaf.branchName;
			text = text.replaceAll(treeNode.branchName, "");
			text = createSeparators(treeNode.branchName.length()) + text;
			writeTreeNodeNavigationNode(writer, leaf, mode, evaluation, treeNavigation, text, query);
		}

	}
	
	private String createSeparators(int length){
		StringBuffer sb = new StringBuffer(" ");
		for(int i = 0; i < length-3; i++){
			sb.append("-");
		}
		sb.append("> ");
		return sb.toString();
	}
	
	private void submittedResults(HttpServletRequest request, PrintWriter writer, String query) throws IOException{
		String userName = request.getParameter("username");
		int queryNumber = 0;
		int numberOfNewsStoriesRegular = 0;
		int numberOfNewsStoriesImproved = 0;
		try {
			queryNumber = Integer.parseInt(request.getParameter("query_number"));
		} catch (NumberFormatException e) {
			writer.append("query number not selected");
			return;
		}
		
		try {
			numberOfNewsStoriesImproved = Integer.parseInt(request.getParameter("improved_number"));
			numberOfNewsStoriesRegular = Integer.parseInt(request.getParameter("regular_number"));
		} catch (NumberFormatException e) {
			writer.append("number of unique queries not selected");
			return;
		}
		
		String overview = request.getParameter("overview");
		String navigate = request.getParameter("navigate");
		
		int improvedResultCount = Integer.parseInt(request.getParameter("improved_result_count"));
		
		EvaluationInfo ei = new EvaluationInfo(userName, query, queryNumber, overview, navigate, numberOfNewsStoriesRegular, numberOfNewsStoriesImproved);
		
		for(int i = 1; i < improvedResultCount+1; i++){
			String field = request.getParameter("score"+i);
			if(field == null){
				writer.append("not all documents evaluated, go back and fill in all fields");
				System.out.println(i);
				return;
			}
			ei.improvedSearch.add(field);
		}
		
		if(userName != null && overview != null && navigate != null){
			if(!"".equals(userName)){
				evaluation.addResults(ei);
				writer.append("results have succesfully been written to disk \n use <div><a href=\"/?results_for_user_name="+userName+"\" target=\"_blank\"> url </a></div> to check that all evaluation results have been recorded");
			} else {
				writer.append("username missing");
			}
		} else {
			writer.append("one or more fields are missing, please fill in all");
		}
	}
	
	private void writerResultsForUserName(PrintWriter writer, String userName){
		HashMap<Integer, EvaluationInfo> queryMap = evaluation.getResults(userName);
		writer.append(userName+"<br>");
		
		if(queryMap == null){
			writer.append("no results found for user " + userName);
		} else {
			for(Map.Entry<Integer, EvaluationInfo> query : queryMap.entrySet()){
				EvaluationInfo ie = query.getValue();
				writer.append("query number "+ie.queryNumber+"<br>");
				writer.append("navigate "+ie.navigate+"<br>");
				writer.append("overview "+ie.overview+"<br>");
				
				writer.append("unique stories in regular serach: " + ie.regularNumber + "<br>");
				
				writer.append("improved search<br>");
				for(int i = 0; i < ie.improvedSearch.size(); i++){
					String field = ie.improvedSearch.get(i);
					writer.append((i + 1) + " : " + field + "<br>");
				}
				writer.append("unique stories in improved serach: " + ie.improvedNumber + "<br>");
			}
		}
	}
}
